#include "TepEditTalk.h"

/* Programmer Name: Nathaniel Cleveland
 * Date Created: Feb 20th, 2001
 * Program Updated by:
 *								Jon Olson, Feb 27, 2001 - Changed data types of TepDBCon
 *
 * Program Description:			Loops through and fills a linked list of stuff.
 *
 * Program dependacies:			fnDefToHex, fnEditChunker
 * Takes:						TepConvData **ppListRoot,	the List Root which I'll link to
 *								char pFileToOpen[80],		the file name i want to open
 * Gives:						The BOOL value FAILURE (-1), if it doesn't work
 *								BOOL value TRUE, if it does.
 */
BOOL fnTepEditTalk(TepDBConvData **ppListRoot, char pFileToOpen[80])
{
	TepDBConvData	*pCurrNode = *ppListRoot;				/* point to the first item in the list*/
    TepDBConvData	*pNewNode = NULL;						/* pointer to the newly created node */
	FILE			*FileHandle = NULL;						/* handle to the file I'm going to open */
	BOOL			bErrorCheck = TRUE;						/* varible to hold bool results */
	char			cpString[ARRAYSIZE];					/* string filled with file chunks */
	char			cpMemString[LONG_ARRAYSIZE];			/* used for long strings that aren't appreate for an array */

	memset(cpMemString, 0, LONG_ARRAYSIZE);				/* zero out the string(s) */
	memset(cpString, 0, ARRAYSIZE );

	/********************************************************************************/
	/**** 1. malloc mem for the list root, fail if we can't CALLER NOW DOES THIS ****/
	/********************************************************************************/

	/****************************************************************************/
	/**** 2. open the file, do a quick read to make sure the file is there ******/
	/****************************************************************************/
	if( (FileHandle  = fopen(pFileToOpen, "r" )) == NULL )
	{
      printf("Failed to open the file\n");
	  return FAILURE;
	}


	/*********************************/
	/**** 3. fill in the stuct(s) ****/
	/*********************************/
	/* loop thru and fill the structs. Stop filling whenever we encounter the string END OF FILE */
	for( ; ; )
	{

		pNewNode = calloc(1, sizeof(TepDBConvData));					/* allocate memory for the new node */

		if(pNewNode == NULL)										/* check for failure */
		{
			printf("failed to calloc memory\n");
			return FAILURE;
		}

		pNewNode->NEXT = NULL;										/* set the next node pointer to NULL */

		if(fnEditChunker(cpString, FileHandle, '\t') == FALSE)		/* get the next chunk of info, check for END OF FILE */
		{
			printf("break\n");
			break;
		}/* end if */

		/* fill in the id, check for the use of #defs */
		if(atoi(cpString) == 0)										/* If this is a #def name, call fnDefToHex */
		{															/*  to convert it to its number. */
			pNewNode->id = fnDefToHex(cpString);					/*   */
			memset(cpString, 0, strlen(cpString) );					/*  zero out the string */
			bErrorCheck = fnEditChunker(cpString, FileHandle, '\t');/* call for the unique number */
			pNewNode->id = pNewNode->id + atoi(cpString);			/* add it to the general class */
		}/* end if */
		else														/* If it is just a number use that */
		{
			pNewNode->id = atoi(cpString);
		}/* end else */

//		printf("%d", pNewNode->id);									/* read what we got */
		memset(cpString, 0, strlen(cpString) );						/* zero out the string */
		if(fnEditChunker(cpString, FileHandle, '\t') == FALSE)		/* get the next chunk of info, check for fail*/
		{
			break;
		}

		/* fill in the attachedto */
		if(atoi(cpString) == 0)										/* If this is a #def name, call fnDefToHex */
		{
			pNewNode->attachedto = fnDefToHex(cpString);
		}
		else														/* If it is just a number use that */
		{
			pNewNode->attachedto = atoi(cpString);
		}

		memset(cpString, 0, strlen(cpString) );						/* zero out the string */
		if(fnEditChunker(cpString, FileHandle, '\t') == FALSE)		/* get the next chunk of info, check for END OF FILE*/
		{
			break;
		}

		/* fill in the owner */
		if(atoi(cpString) == 0)										/* If this is a #def name, call fnDefToHex */
		{
			pNewNode->owner = fnDefToHex(cpString);
		}
		else														/* If it is just a number use that */
		{
			pNewNode->owner = atoi(cpString);
		}

		/* get details chunk */
		if(fnEditChunker(cpMemString, FileHandle, '\n') == FALSE)		/* get the next chunk of info, check for END OF FILE */
		{
			break;
		}

		/* fill in the details */
		pNewNode->details = calloc( strlen(cpMemString) + 1, sizeof(char));	/* calloc mem for the string */

		if(pNewNode->details == NULL)										/* check for error in calloc */
		{
			printf("calloc didn't alloc mem for the details.\n");
			return FAILURE;
		}
											/* copy the string to the memory alloced string */
		pNewNode->details = memmove(pNewNode->details, cpMemString, strlen(cpMemString) );

		/* Test the node(s) */
/*
		printf("0x%x\n", pNewNode->id);
		printf("0x0%x\n", pNewNode->attachedto);
		printf("0x0%x\n", pNewNode->owner);
		printf("%s\n", pNewNode->details);
*/
		/* set this node to the list */
		pCurrNode->NEXT = pNewNode;

		/* point CurrNode at this node */
		pCurrNode = pNewNode;
	}/* end for */


	if( fclose(FileHandle) != 0)
	{
		printf("damn file wouldn't close");
	}

	return TRUE;
}/* End fnTepEditTalk */

/*************** DEBUG MAIN **************/
/*
int main(void)
{
	char inputArray[255];
	BOOL bCheckBool;
	TepConvData *pListRoot=NULL;

	pListRoot = calloc(1, sizeof(TepChar));

	while(1)
	{
		if(gets(inputArray) == NULL)
			break;

		if(inputArray == 0)
			break;

		bCheckBool = fnTepEditTalk(&pListRoot, "TalkContent.txt");
		if(bCheckBool == FALSE)
		{
			printf("\nFailed\n");
		}
		else
		{
			printf("\nDidn't fail\n");
		}
	}
	free(pListRoot);
	return 0;
}
*/

int main ( int argc, char *argv[], char *envp[])
{
    pTepDBConvData pListRoot = NULL;

    if (argc > 1)
    {
        BOOL bCheckBool;

        pListRoot = calloc(1, sizeof(TepDBConvData));
        bCheckBool = fnTepEditTalk(&pListRoot, argv[1]);

        if(bCheckBool == FAILURE || bCheckBool == FALSE)
        {
            printf("\nFailed\n");
        }
        else
        {
            printf("\nDidn't fail\n");
        }
    }
    else
    {
        printf("Usage:\n  TepEditTalk <filename> [outputfile]\n    Where <filename> is the name of the file to be processed.\n    Outputfile is the name of the file to write to.\n\n");
    }

    if (pListRoot != NULL)
    {
        FILE *ioStream = NULL;

        if (argc > 2)
        {
            ioStream = fopen(argv[2], "w" );
        }
        else
        {
            ioStream = fopen("tepTalk.dat", "w");
        }

        if (ioStream != NULL)
        {
            fnTepEditWriteFile(TEP_TALK, pListRoot->NEXT, ioStream);
            fclose(ioStream);
        }
        else
        {
            printf("There was an error in opening the output file.\n");
        }

        //fnTepCharDelete(&pListRoot, NULL);
    }

    return 0;
}
