#include "TepEditUtils.h"

/* Programmer Name: Nathaniel Cleveland
 * Date Created: Feb 20th, 2001
 * Program Updated by:          Jon Olson - Feb 25, 2001 - Added saving functionality
 *
 * Program Description:         A collection of utilites for The TepEdit functionettes.
 *
 * Program dependacies:         CommandDefines.h, CharacterDefines.h
 */



/*FUNCTION:     fnEditChunker
 *PARAMETERS:   char *cpString              the string you want filled
 *              FILE *FileHandle            the file you want to look thru
 *              char cTerminator            what character you want to signafy the end of your string
 *RETURN VALUE: BOOLEAN TRUE                It did what it needed to do
 *              BOOLEAN FALSE               It found the END OF STRING
 *SIDE EFFECTS: moves the file pointer to the file handle that is passed
 *DESCRIPTION:  This function hacks through and pulls a string from a file, it'll stop when it sees the cTerminator
 *              character.
*/
BOOL fnEditChunker(char *cpString, FILE *FileHandle, char cTerminator)
{
    int     i = 0;                          /* counter */
    char    cComp = '\0';                   /* compairson checker */


    /* loop through and get the id */
    /* while the current char isn't a tab, keep filling the string */
    /* get the next character, inc the i */
    for(i = 0, cComp = getc(FileHandle) ; cComp != cTerminator ; cComp = getc(FileHandle), i++)
    {
        cpString[i] = cComp;                /* put this character in the array */
    }/* end for */

    cpString[i] = '\0';                     /* null terminate the string */

    if(strcmp(cpString, "END OF FILE") == 0)
    {
        return FALSE;
    }/* end if */
    return TRUE;
} /* end fnEditChunker */

/*FUNCTION:     fnTepEditUnNodeLink
 *PARAMETERS:   TepIDUnNode **ppListRoot    the root you want the nodes attached to
 *              FILE *FileHandle            the file you want to look thru
 *              char cComp                  The Character you want to signify the end of nodes
 *RETURN VALUE: BOOLEAN TRUE                It did what it needed to do
 *              BOOLEAN FALSE               It found an error somewhere, most likely at allocing
 *SIDE EFFECTS: moves the file pointer to the file handle that is passed
 *DESCRIPTION:  This function hacks through and pulls a string from a file, adds it to a linked list of them
 *              it'll stop when it sees the cComp character as the next character.
*/
BOOL fnTepEditUnNodeLink(TepIDUnNode **ppListRoot, FILE *FileHandle, char cComp)
{
    TepIDUnNode     *pCurrNode = *ppListRoot;               /* point to the first item in the list*/
    TepIDUnNode     *pNewNode = NULL;                       /* pointer to the newly created node */
    BOOL            bErrorCheck = TRUE;                     /* varible to hold bool results */
    char            cpString[SIZE_OF_ARRAY];                    /* string filled with file chunks */

    memset(cpString, 0, SIZE_OF_ARRAY );                        /* zero out the string */

    for( ; ; )
    {

        pNewNode = calloc(1, sizeof(TepIDUnNode));                  /* allocate memory for the new node */

        if(pNewNode == NULL)                                        /* check for failure */
        {
            printf("failed to calloc memory for the TepIDUnNode\n");
            return FALSE;
        }

        pNewNode->NEXT = NULL;                                      /* set the next node pointer to NULL */

        if(fnEditChunker(cpString, FileHandle, ',') == FALSE)       /* get the next chunk of info, check for END OF FILE */
        {
            printf("break\n");
            break;
        }/* end if */
        /* fill in the id, check for the use of #defs */

//      printf("Attach this: %s\n", cpString);

        if(atoi(cpString) == 0)                                     /* If this is a #def name, call fnDefToHex */
        {                                                           /*  to convert it to its number. */
            pNewNode->NODE = fnDefToHex(cpString);
        }/* end if */
        else                                                        /* If it is just a number use that */
        {
            pNewNode->NODE = atoi(cpString);
        }/* end else */

        /* Test the node(s) */
		//printf("%08X or %d\n", pNewNode->NODE, pNewNode->NODE);       /* see what we've got */

        pCurrNode->NEXT = pNewNode;                                 /* set this node to the list */
        pCurrNode = pNewNode;                                       /* point CurrNode at this node */

        if(getc(FileHandle) == cComp)                               /* pull out the next character in the file */
        {
            break;                                                  /* break if it matches the cComp */
        }/* end if */

        /* if not move the file pointer back by one */
        if(fseek(FileHandle, -1, SEEK_CUR ) != 0 )              /* check for failure to reset pointer */
        {
            printf("failed to reset the file pointer in fnTepEditUnNodeLink\n");
            return FALSE;
        }
    }/* end for */

    return TRUE;
}

/*FUNCTION:     fnEditChunker
 *PARAMETERS:   char *cpString              #define name you're looking for
 *RETURN VALUE: TepID                       the #define value of what you sent in or 0 for nothing
 *                                          ie "totok" would return "TEP_CHARACTER + CHAR_TOTOK". etc
 *SIDE EFFECTS: none
 *DESCRIPTION:  Hacks through and uses strcmp to compair names until it finds what it is looking for.
 *              It may be chaunky but it does the job...
*/
TepID fnDefToHex(char *cpString)
{
    int i = 0;                          /* counter */

    /* convert the string to all upper case */
    for(i = 0 ; cpString[i] != 0 ; i++)
    {
        cpString[i] = toupper(cpString[i]);
    }

    /* use strcmp to compair the string to all character names in else if statements, */
    /* if there is a match return that character's TepID */
/*************************BEGIN TEP_CHARACTER *************************/
    if(strcmp(cpString, "ALAQUA" ) == 0)
    {
        return TEP_CHARACTER +  CHAR_ALAQUA;
    }/* end if */
    else if(strcmp(cpString, "AMALIYA") == 0)
        {
            return TEP_CHARACTER +  CHAR_AMALIYA;
        }/* end else if */
    else if(strcmp(cpString, "ANOKI") == 0)
        {
            return TEP_CHARACTER +  CHAR_ANOKI;
        }/* end else if */
    else if(strcmp(cpString, "COMAN") == 0)
        {
            return TEP_CHARACTER +  CHAR_COMAN;
        }/* end else if */
    else if(strcmp(cpString, "FIALA") == 0)
        {
            return TEP_CHARACTER +  CHAR_FIALA;
        }/* end else if */
    else if(strcmp(cpString, "HANIA") == 0)
        {
            return TEP_CHARACTER +  CHAR_HANIA;
        }/* end else if */
    else if(strcmp(cpString, "IMALA") == 0)
        {
            return TEP_CHARACTER +  CHAR_IMALA;
        }/* end else if */
    else if(strcmp(cpString, "IOAKIM") == 0)
        {
            return TEP_CHARACTER +  CHAR_IOAKIM;
        }/* end else if */
    else if(strcmp(cpString, "ISAK") == 0)
        {
            return TEP_CHARACTER +  CHAR_ISAK;
        }/* end else if */
    else if(strcmp(cpString, "KAILA") == 0)
        {
            return TEP_CHARACTER +  CHAR_KAILA;
        }/* end else if */
    else if(strcmp(cpString, "KALIL") == 0)
        {
            return TEP_CHARACTER +  CHAR_KALIL;
        }/* end else if */
    else if(strcmp(cpString, "KAVERI") == 0)
        {
            return TEP_CHARACTER +  CHAR_KAVERI;
        }/* end else if */
    else if(strcmp(cpString, "LEILANI") == 0)
        {
            return TEP_CHARACTER +  CHAR_LEILANI;
        }/* end else if */
    else if(strcmp(cpString, "MERRIPEN") == 0)
        {
            return TEP_CHARACTER +  CHAR_MERRIPEN;
        }/* end else if */
    else if(strcmp(cpString, "QIANRU") == 0)
        {
            return TEP_CHARACTER +  CHAR_QIANRU;
        }/* end else if */
    else if(strcmp(cpString, "RUSHI") == 0)
        {
            return TEP_CHARACTER +  CHAR_RUSHI;
        }/* end else if */
    else if(strcmp(cpString, "SENWE") == 0)
        {
            return TEP_CHARACTER +  CHAR_SENWE;
        }/* end else if */
    else if(strcmp(cpString, "TOTOK") == 0)
        {
            return TEP_CHARACTER + CHAR_TOTOK;
        }/* end else if */
    else if(strcmp(cpString, "WINGI") == 0)
        {
            return TEP_CHARACTER +  CHAR_WINGI;
        }/* end else if */
    else if(strcmp(cpString, "SHEROESHA") == 0)
        {
            return TEP_CHARACTER +  CHAR_SHEROESHA;
        }/* end else if */
/*************************BEGIN TEP_TALK *************************/
    else if(strcmp(cpString, "GENINSULT") == 0)
        {
            return TEP_TALK +  TALK_GENINSULT;
        }/* end else if */
    else if(strcmp(cpString, "GENCOMPLIAMENT") == 0)
        {
            return TEP_TALK +  TALK_GENCOMPLIAMENT;
        }/* end else if */
    else if(strcmp(cpString, "HELLO") == 0)
        {
            return TEP_TALK +  TALK_HELLO;
        }/* end else if */
    else if(strcmp(cpString, "GOODBYE") == 0)
        {
            return TEP_TALK +  TALK_GOODBYE;
        }/* end else if */
    else if(strcmp(cpString, "ANSQUERY") == 0)
        {
            return TEP_TALK +  TALK_ANSQUERY;
        }/* end else if */
    else if(strcmp(cpString, "ANSEVIDENCEYES") == 0)
        {
            return TEP_TALK +  TALK_ANSEVIDENCEYES;
        }/* end else if */
    else if(strcmp(cpString, "ANSEVIDENCENO") == 0)
        {
            return TEP_TALK +  TALK_ANSEVIDENCENO;
        }/* end else if */
    else if(strcmp(cpString, "CLUESLOCATION") == 0)
        {
            return TEP_TALK +  TALK_CLUESLOCATION;
        }/* end else if */
    else if(strcmp(cpString, "CLUESCHARACTER") == 0)
        {
            return TEP_TALK +  TALK_CLUESCHARACTER;
        }/* end else if */
    else if(strcmp(cpString, "INIT") == 0)
        {
            return TEP_TALK +  TALK_INIT;
        }/* end else if */
    else if(strcmp(cpString, "GENFAILURE") == 0)
        {
            return TEP_TALK +  TALK_GENFAILURE;
        }/* end else if */
/*************************BEGIN TEP_COMMAND *************************/
    else if(strcmp(cpString, "GO") == 0)
        {
            return TEP_COMMAND +  COM_GO;
        }/* end else if */
    else if(strcmp(cpString, "MOVE") == 0)
        {
            return TEP_COMMAND +  COM_MOVE;
        }/* end else if */
    else if(strcmp(cpString, "WALK") == 0)
        {
            return TEP_COMMAND +  COM_WALK;
        }/* end else if */
    else if(strcmp(cpString, "LEAVE") == 0)
        {
            return TEP_COMMAND +  COM_LEAVE;
        }/* end else if */
    else if(strcmp(cpString, "NORTH") == 0)
        {
            return TEP_COMMAND +  COM_NORTH;
        }/* end else if */
    else if(strcmp(cpString, "SOUTH") == 0)
        {
            return TEP_COMMAND +  COM_SOUTH;
        }/* end else if */
    else if(strcmp(cpString, "EAST") == 0)
        {
            return TEP_COMMAND +  COM_EAST;
        }/* end else if */
    else if(strcmp(cpString, "WEST") == 0)
        {
            return TEP_COMMAND +  COM_WEST;
        }/* end else if */
    else if(strcmp(cpString, "PUT") == 0)
        {
            return TEP_COMMAND +  COM_PUT;
        }/* end else if */
    else if(strcmp(cpString, "GRAB") == 0)
        {
            return TEP_COMMAND +  COM_GRAB;
        }/* end else if */
    else if(strcmp(cpString, "TAKE") == 0)
        {
            return TEP_COMMAND +  COM_TAKE;
        }/* end else if */
    else if(strcmp(cpString, "SWIPE") == 0)
        {
            return TEP_COMMAND +  COM_SWIPE;
        }/* end else if */
    else if(strcmp(cpString, "LOOK") == 0)
        {
            return TEP_COMMAND +  COM_LOOK;
        }/* end else if */
    else if(strcmp(cpString, "EXAMINE") == 0)
        {
            return TEP_COMMAND +  COM_EXAMINE;
        }/* end else if */
    else if(strcmp(cpString, "SEE") == 0)
        {
            return TEP_COMMAND +  COM_SEE;
        }/* end else if */
    else if(strcmp(cpString, "INSPECT") == 0)
        {
            return TEP_COMMAND +  COM_INSPECT;
        }/* end else if */
    else if(strcmp(cpString, "CHECK") == 0)
        {
            return TEP_COMMAND +  COM_CHECK;
        }/* end else if */
    else if(strcmp(cpString, "VIEW") == 0)
        {
            return TEP_COMMAND +  COM_VIEW;
        }/* end else if */
    else if(strcmp(cpString, "HELP") == 0)
        {
            return TEP_COMMAND +  COM_HELP;
        }/* end else if */
    else if(strcmp(cpString, "SAVE") == 0)
        {
            return TEP_COMMAND +  COM_SAVE;
        }/* end else if */
    else if(strcmp(cpString, "LOAD") == 0)
        {
            return TEP_COMMAND +  COM_LOAD;
        }/* end else if */
    else if(strcmp(cpString, "QUIT") == 0)
        {
            return TEP_COMMAND +  COM_QUIT;
        }/* end else if */
    else if(strcmp(cpString, "GREET") == 0)
        {
            return TEP_COMMAND +  COM_GREET;
        }/* end else if */
    else if(strcmp(cpString, "HELLO") == 0)
        {
            return TEP_COMMAND +  COM_HELLO;
        }/* end else if */
    else if(strcmp(cpString, "HI") == 0)
        {
            return TEP_COMMAND +  COM_HI;
        }/* end else if */
    else if(strcmp(cpString, "INSULT") == 0)
        {
            return TEP_COMMAND +  COM_TAUNT;
        }/* end else if */
    else if(strcmp(cpString, "TALK") == 0)
        {
            return TEP_COMMAND +  COM_TALK;
        }/* end else if */
    else if(strcmp(cpString, "SPEAK") == 0)
        {
            return TEP_COMMAND +  COM_SPEAK;
        }/* end else if */
    else if(strcmp(cpString, "ASK") == 0)
        {
            return TEP_COMMAND +  COM_ASK;
        }/* end else if */
    else if(strcmp(cpString, "SHOW") == 0)
        {
            return TEP_COMMAND +  COM_SHOW;
        }/* end else if */
    else if(strcmp(cpString, "GOODBYE") == 0)
        {
            return TEP_COMMAND +  COM_GOODBYE;
        }/* end else if */
    else if(strcmp(cpString, "CHECK") == 0)
        {
            return TEP_COMMAND +  COM_CHECK;
        }/* end else if */
    else if(strcmp(cpString, "DONE") == 0)
        {
            return TEP_COMMAND +  COM_DONE;
        }/* end else if */
    else if(strcmp(cpString, "REST") == 0)
        {
            return TEP_COMMAND +  COM_REST;
        }/* end else if */
    else if(strcmp(cpString, "WAIT") == 0)
        {
            return TEP_COMMAND +  COM_WAIT;
        }/* end else if */
/*************************BEGIN TEP_CLUE *************************/
    else if(strcmp(cpString, "PERPITEM") == 0)
        {
            return TEP_CLUE + CLUE_PERPITEM;
        }/* end else if */
    else if(strcmp(cpString, "PERPLOCA") == 0)
        {
            return TEP_CLUE + CLUE_PERPLOCA;
        }/* end else if */
    else if(strcmp(cpString, "PERPVERBALSIGHT") == 0)
        {
            return TEP_CLUE + CLUE_PERPVERBALSIGHT;
        }/* end else if */
    else if(strcmp(cpString, "PERPVERBALSOUND") == 0)
        {
            return TEP_CLUE + CLUE_PERPVERBALSOUND;
        }/* end else if */
    else if(strcmp(cpString, "FROMDEATHONPERP") == 0)
        {
            return TEP_CLUE + CLUE_FROMDEATHONPERP;
        }/* end else if */
    else if(strcmp(cpString, "FROMDEATHATLOCA") == 0)
        {
            return TEP_CLUE + CLUE_FROMDEATHATLOCA;
        }/* end else if */
    else if(strcmp(cpString, "TYPEDEATHONPERP") == 0)
        {
            return TEP_CLUE + CLUE_TYPEDEATHONPERP;
        }/* end else if */
    else if(strcmp(cpString, "TYPEDEATHATLOCA") == 0)
        {
            return TEP_CLUE + CLUE_TYPEDEATHATLOCA;
        }/* end else if */

    return 0;
}/* end function fnDefToHex */


/*FUNCTION:     fnTepEditWriteFile
 *PARAMETERS:   DWORD dwType                #defined type of file you are writing
 *              void *pListRoot             pointer to the ListRoot
 *              FILE *outFile               this is the file handle used for the output of the generated file. Pass NULL to print to screen.
 *RETURN VALUE: BOOL                        SUCCESS/FAILURE
 *SIDE EFFECTS: data file generated
 *DESCRIPTION:  Based on the type given, the function reads the entire dataset and prints it to file.
*/
BOOL fnTepEditWriteFile(const DWORD dwType, void *pListRoot, FILE *outFile)
{
    BOOL fRet = FAILURE;

    if (pListRoot != NULL)
    {
        void *pCurNode = pListRoot;
        outFile = (outFile) ? (outFile) : (stdout);
        while (pCurNode != NULL)
        {
            static DWORD dwWriteZero = 0;
            switch (dwType)
            {
                case TEP_CLUE:
                {
                    void *lpvTmp = NULL;

                    /* write the id to the file */
                    fwrite(&((pTepClueData)pCurNode)->id, sizeof(((pTepClueData)pCurNode)->id), 1, outFile);

                    /* write the attached to list to the file */
                    fwrite(&((pTepClueData)pCurNode)->attachedto, sizeof(((pTepClueData)pCurNode)->attachedto), 1, outFile);

                    /* write the null-terming string with the details */
                    fprintf(outFile, "%s%c", ((pTepClueData)pCurNode)->details,0);
                    //fwrite(&((pTepClueData)pCurNode)->details, sizeof(((pTepClueData)pCurNode)->details[0]), strlen(((pTepClueData)pCurNode)->details) + 1, outFile);

                    pCurNode = ((pTepClueData)pCurNode)->NEXT;
                    break;
                }

                case TEP_ROOM:
                {
                    printf("Room handling functionality not implemented.\n");

                    pCurNode = NULL;
                    break;
                }

                case TEP_DEATH:
                {
                    /* write the id to the file */
                    fwrite(&((pTepDeath)pCurNode)->id, sizeof(((pTepDeath)pCurNode)->id), 1, outFile);

                    /* write the location to the file */
                    fwrite(&((pTepDeath)pCurNode)->location, sizeof(((pTepDeath)pCurNode)->location), 1, outFile);

                    /* write the null-terming string with the details */
                    fprintf(outFile, "%s%c", ((pTepDeath)pCurNode)->desc,0);
                    //fwrite(&((pTepDeath)pCurNode)->desc, sizeof(((pTepDeath)pCurNode)->desc[0]), strlen(((pTepDeath)pCurNode)->desc) + 1, outFile);

                    pCurNode = ((pTepDeath)pCurNode)->NEXT;
                    break;
                }

                case TEP_TALK:
                {
                    /* write the id to the file */
                    fwrite(&((pTepDBConvData)pCurNode)->id, sizeof(((pTepDBConvData)pCurNode)->id), 1, outFile);

                    /* write the attachment to the file */
                    fwrite(&((pTepDBConvData)pCurNode)->attachedto, sizeof(((pTepDBConvData)pCurNode)->attachedto), 1, outFile);

                    /* write the owner to the file */
                    fwrite(&((pTepDBConvData)pCurNode)->owner, sizeof(((pTepDBConvData)pCurNode)->owner), 1, outFile);

                    /* write the null-terming string with the details */
                    fprintf(outFile, "%s%c", ((pTepDBConvData)pCurNode)->details,0);
                    //fwrite(&((pTepDBConvData)pCurNode)->details, sizeof(((pTepDBConvData)pCurNode)->details[0]), strlen(((pTepConvData)pCurNode)->details) + 1, outFile);

                    pCurNode = ((pTepConvData)pCurNode)->NEXT;
                    break;
                }

                case TEP_CHARACTER:
                {
                    void *lpvTmp = NULL;

                    /* write the id to the file */
                    fwrite(&((pTepChar)pCurNode)->id, sizeof(((pTepChar)pCurNode)->id), 1, outFile);
                    printf("\n[%08X]\n", ((pTepChar)pCurNode)->id);

                    /* write the null-terming string with the name */
                    fprintf(outFile, "%s%c", ((pTepChar)pCurNode)->name,0);
                    printf("%s%c\n", ((pTepChar)pCurNode)->name,0);
                    //fwrite(((pTepChar)pCurNode)->name, sizeof(((pTepChar)pCurNode)->name[0]), strlen(((pTepChar)pCurNode)->name) + 1, outFile);

                    /* write the null-terming string with the details */
                    fprintf(outFile, "%s%c", ((pTepChar)pCurNode)->desc,0);
                    printf("%s%c\n", ((pTepChar)pCurNode)->desc,0);
                    //fwrite(&((pTepChar)pCurNode)->desc, sizeof(((pTepChar)pCurNode)->desc[0]), strlen(((pTepChar)pCurNode)->desc) + 1, outFile);

                    /* write the attached to list to the file */
                    lpvTmp = ((pTepChar)pCurNode)->locations;
                    while (lpvTmp != NULL)
                    {
						if (((pTepIDUnNode)lpvTmp)->NODE != 0)
						{
                        	fwrite(&((pTepIDUnNode)lpvTmp)->NODE, sizeof(((pTepIDUnNode)lpvTmp)->NODE), 1, outFile);
                        	printf("%08X={%08X,%08X}\n", lpvTmp, ((pTepIDUnNode)lpvTmp)->NODE, ((pTepIDUnNode)lpvTmp)->NEXT);
						}
                        lpvTmp = ((pTepIDUnNode)lpvTmp)->NEXT;
                    }
                    /* write the marker for end of attachment list */
                    fwrite(&dwWriteZero, sizeof(dwWriteZero), 1, outFile);
                    printf("%08X//\n",0);

                    pCurNode = ((pTepChar)pCurNode)->NEXT;
                    break;
                }

                default:
                    break;
            }
        }
    }

    return fRet;
}
