/* Program ID:          tep_ask.c
 * Programmer Name:     Matt Phillips
 * Date Created:        February 17, 2001
 * Program Updated by:  Matt Phillips       Date of Update: 02/17/01
 * Program Description: tep_ask.c handles conversational answers to questions
 *                      the player asks to game characters
 * Dependencies:        "tep_incs.h", tep_ask.h
 * Input Files:         N/A
 * Output Files:        N/A
 */
#include "tep_incs.h"



/* Function:            fnAsk
 * Parameters:          TepCurState *   pointer to the gamestate struct
 *                      TepChar *       pointer to the character the player is talking too
 *                      TepParse *      pointer to the command struct
 * Return Value:        0 (for now...)
 * Side Effects:        outputs the answer to a question
 * Description:         finds an answer to a question that was asked and prints the answer.
 *                      since the player can only ask about rooms and other characters it's easy.
 */
BOOL fnAsk( TepCurState *pGameState, TepChar *pSpeaker, TepParse *pCmd )
{
    TepConvData *pAnswer = NULL;    /* this is a pointer to the answer to the question */

    /* interpret the command */
    /* is it a question about a character? */
    if( pCmd->Char2 != PARSE_UNUSED )
    {
        /* WTF!?! HAPPENS IF YOU ASK THE CHARACTER ABOUT THEMSELVES??????? */

        /* if the character isn't dead... */
        if( fnTepCharSearch( pGameState->liveList, pCmd->Char2 ) != NULL )
        {
            /* what do we have to say about them? */
            pAnswer = fnTepConvDataSearch( pSpeaker->conv, TEP_TALK | TALK_ANSQUERY, pCmd->Char2 );
        }
        else
        {
            /* we don't speak of the dead here */            
            pAnswer = fnTepConvDataSearch( pGameState->genConv, TEP_TALK | TALK_TALKABOUTDEAD, 0 );
        }
    }
    /* is it a question about a location? */
    else if( pCmd->Reserved != PARSE_UNUSED )
    {
        /* it's a location - find the answer */
        /* is it a location that they frequent? */
        if( fnTepIDSearch( pSpeaker->locations, pCmd->Reserved ) != NULL )
        {
            /* Then they have something unique to say about it */
            pAnswer = fnTepConvDataSearch( pSpeaker->conv, TEP_TALK | TALK_CLUESLOCATION, pCmd->Reserved );
        }
        else
        {
            /* use the generic description of the location */
            pAnswer = fnTepConvDataSearch( pGameState->genConv, TEP_TALK | TALK_CLUESLOCATION, pCmd->Reserved );
        }
    }
    /* what the hell is the player talking about? */
    else
    {
        /* answer with an "I don't know" response */
        // What specifies the unknown answer? is it generic? (ie. TALK_NOANSWER)
        pAnswer = fnTepConvDataSearch( pGameState->genConv, TEP_TALK | TALK_GENFAILURE, 0 );
    }

    /* output of pAnswer if it's not NULL... */
    if( pAnswer != NULL )
    {
		//nate 4/10/01
        fnPrintOut(NOPAUSE,NULL,NULL, 1, pAnswer->details );
    }

    return TRUE;
}
