#include "tep_incs.h"

/*** Find the gender of a character for fnPrintOut ***/
unsigned int CharGenderFind(TepID CharID)
{
	CharID &= 0x00000fff;				// Strip out all but the character's private ID

	// If this is true return MALE for fnPrintOut
	if(CharID <= CHAR_WINGI)				
		return MALE;
	
	// Else return 0 for female
	return 0;
}




/*FUNCTION:     fnListPrint
 *PARAMETERS:   _TepCurState    Used to obtain access to lists with descriptions
                TepIDUnNode     RootNode to search.
                                NOTE: To display the contents of a room without moving
                                      the cmd member of the TepParse should be DISPLAY.
 *RETURN VALUE: none
 *SIDE EFFECTS: Prints description data to the screen using fnPrintOut
 *DESCRIPTION:  This function is intended to be used by DisplayPlayerLocation
 *              to print out the descriptions of dynamic objects in a room.
 */
void fnListPrint(TepCurState *pGameState, TepIDUnNode *pListRoot)
{
    TepIDUnNode *pCurrNode;

	printf("\n\n");
    /*Go through the list of objects*/
    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        /*  Use a bit mask on the TepID's to determine what sort of object we have
            and search through the appropriate lists from the game state for the
            descriptions*/
        switch(pCurrNode->NODE & 0xff000000)
        {
/*			printf("\n***FOUND SOMETHING***\n");
*/            case TEP_CHARACTER:
                {
                    TepChar *pDude;
/*					printf("\n***Found Tep Character***\n");
*/
                    if((pDude = fnTepCharSearch(pGameState->liveList, pCurrNode->NODE)) != NULL)
					{
//						printf("Character is in the live list\n");
		                fnPrintOut(NOPAUSE, NULL,NULL,1, pDude->name);
						printf(" is here.\n");
				
					}
					else
					{
/*						printf("Character is not in the live list\n");
*/					}
					break;
                }
            case TEP_DEATH:
                {
//					TepDataString *pVictim;
//
//					if((pVictim = fnTepDataStringSearch(pGameState->keywords, pGameState->currentMurder->victim)) == NULL)
//						break;
//
//					/* Same prob as below */
//                    fnPrintOut(NOPAUSE, NULL,pVictim->data,1, pGameState->currentMurder->death->desc);
					break;
                }
            case TEP_CLUE:
                {
					//TepDataString *fnTepDataStringSearch(TepDataString *pListRoot, TepID searchID)
                    TepClueData *pClue;
					TepDataString *pVictim;
                    pClue = fnClueDataSearch(pGameState->currentMurder->death->clues, pCurrNode->NODE, 0 );

					if((pVictim = fnTepDataStringSearch(pGameState->keywords, pGameState->currentMurder->victim)) == NULL)
						break;

					/* Nate isn't sure whether the 3rd param of fnPrintOut should be victim or murderer. depends
					on how the content was written. He thinks victim is more common. 4/10/01 */
					if(pClue != NULL)
						fnPrintOut(NOPAUSE, pClue->keyword,pVictim->data,2, "\n", pClue->details);
					break;
                }
        } /*end switch*/
    }/*end for*/
}/*end List Print*/



/*FUNCTION:     DisplayPlayerLocation
 *PARAMETERS:   _TepCurState    Used to determine current location and update to new location
                TepParse        Used to determine where to move
                                NOTE: To display the contents of a room without moving
                                      the cmd member of the TepParse should be DISPLAY.
 *RETURN VALUE: Success or Failure
 *SIDE EFFECTS: Changes the game state to the new location.
                Calls PrintOut to display text for new location.
 *DESCRIPTION:  This fucntion will be passed a TepParse struct which
                will contain a direction or a location id. The function
                will determine whether or not moving in that location
                is possible. If movement in that direction is possible
                it will display text describing the new location and
                update the current game state.
 */
BOOL fnDisplayPlayerLocation(TepCurState *pGameState, TepParse msg)
{
    TepRoom			*room = NULL;      /*Used to obtain current room from game state*/
    TepRoom			*newRoom	= NULL;   /*Used to determine new room*/
    TepDynRoom		*pDynRoom = NULL;
	TepDataString	*pVictim;
	TepCurMurder	*pCurDeath = NULL;
	char			KillerNumber[2] = {0};	/* set string to null, added by nate 4/11/01 */
	TepIDUnNode		*IDKillerList = NULL;	/* added by nate 4/11/01 */
	int				i = 0;
/*
	printf("Clue: %x\t Char1: %x\t Char2: %x\t Comm: %x\tRes: %x\t\n", 
		msg.Clue, msg.Char1, msg.Char2, msg.Command, msg.Reserved);
*/
	/** Find the current murder struct for printout **/
	for(pCurDeath = pGameState->currentMurder ;pCurDeath->NEXT != NULL ; pCurDeath = pCurDeath->NEXT );
	
//	printf("\nMurder is here>> 0x%X\n", pCurDeath->death->location);
	printf("\n");
	
	/*******************************************************************************************************/
	/****************************************** Added by nate 4/10/01 **************************************/
	if((pVictim = fnTepDataStringSearch(pGameState->keywords, pCurDeath->victim)) == NULL)
	{
		pVictim->data = "<--victim-->";
		pVictim->id = pCurDeath->victim;
		pVictim->NEXT = NULL;
	}

	/* find out how many statues are left by counting how many killers are still alive */				
	for(i = 0,	IDKillerList = pGameState->killersList; IDKillerList != NULL; IDKillerList = IDKillerList->NEXT)
	{
		if(fnTepCharSearch(pGameState->liveList, IDKillerList->NODE) != NULL)
		i++;		
	}
	/* convert that number to a string so we know how many statues there we're before the killing */
	_itoa(i, KillerNumber, 10);
	/*******************************************************************************************************/
	/*******************************************************************************************************/
    #if defined(_DEBUG)
    //printf("pGameState = %08X\n", pGameState);
    //printf("pGameState->currentRoom = %08X\n", pGameState->currentRoom);
	printf("\n\nCommand: %x\tRoom ID: %x\n", msg.Command, msg.Reserved);
	#endif

    room = pGameState->currentRoom; // pointers to pointers? or just pointers?

    /*If the command is possible then the room is changed*/
    if (room == NULL)
    	return FAILURE;
    if((room->cmd.allCmd) & (msg.Command))
    {
		/*Print out information if that is all that is wanted*/
		if((msg.Command & COM_DISPLAY)== COM_DISPLAY)
		{
			//fnPrintOut(NOPAUSE, NULL,NULL,1, pGameState->currentRoom->name);
			fnPrintOut(NOPAUSE, KillerNumber,NULL,2, "\n", pGameState->currentRoom->desc);
			/*Obtain access to the dynamic data and call fnListPrint to print out descriptions*/
			if((pDynRoom = fnTepDynRoomSearch(pGameState->mapLite, pGameState->currentRoom->id)) != NULL)
			{
				fnListPrint(pGameState, pDynRoom->people);//pGameState->currentRoom->objs); //was objs
				
#if 1
				/******************************** Added by Nate 4/12/01  ********************************/		
				/* PrintOut the murder if this is the right place */
				if(pGameState->currentRoom->id == pCurDeath->death->location)
					fnPrintOut(NOPAUSE | CharGenderFind(pCurDeath->victim), KillerNumber, pVictim->data,2, "\n", pCurDeath->death->desc);
				/*******************************************************************************/
#endif
				/** PrintOut all the clues in this room if this is the right place Added by nate,  4/7/01 **/
				if( (pDynRoom = fnTepDynRoomSearch(pGameState->mapLite, pGameState->currentRoom->id)) != NULL)
					fnListPrint(pGameState, pDynRoom->objs);
				/*******************************************************************************************/		
				
			}
		}/*end if*/
        else if( (msg.Command & COM_GO) == COM_GO)
        {
            if((msg.Reserved & TEP_ROOM) != TEP_ROOM)
            {
                switch(msg.Reserved)
                {
                case (TEP_COMMAND | COM_NORTH):
					//printf("Goin' north %x\n", msg.Reserved);
                    newRoom = fnTepRoomSearch(pGameState->FullMap, room->rooms->NODE);
					//printf("%x\n", room->rooms->NODE);
                    break;
                case (TEP_COMMAND | COM_EAST):
					//printf("Goin' east %x\n", msg.Reserved);
                    newRoom = fnTepRoomSearch(pGameState->FullMap, room->rooms->NEXT->NODE);
                    break;
                case (TEP_COMMAND | COM_SOUTH):
					//printf("Goin South %x\n", msg.Reserved);
                    newRoom = fnTepRoomSearch(pGameState->FullMap, room->rooms->NEXT->NEXT->NODE);
                    break;
                case (TEP_COMMAND | COM_WEST):
					//printf("Goin west\n");
                    newRoom = fnTepRoomSearch(pGameState->FullMap, room->rooms->NEXT->NEXT->NEXT->NODE);
                    break;
                }/* end switch */

                if(newRoom != NULL)
                {
                    /*Set the new room as the current room in the game state*/
                    pGameState->currentRoom = newRoom;
                    //fnPrintOut(NOPAUSE, NULL,NULL,1, pGameState->currentRoom->name);
                    fnPrintOut(NOPAUSE, KillerNumber,NULL,2, "\n", pGameState->currentRoom->desc);

#if 1
				/******************************** Added by Nate 4/12/01  ********************************/		
				/* PrintOut the murder if this is the right place */
				if(pGameState->currentRoom->id == pCurDeath->death->location)
					fnPrintOut(NOPAUSE | CharGenderFind(pCurDeath->victim), KillerNumber, pVictim->data,2, "\n", pCurDeath->death->desc);
				/*******************************************************************************/
#endif
					/** PrintOut all the clues in this room if this is the right place Added by nate,  4/7/01 **/
					if( (pDynRoom = fnTepDynRoomSearch(pGameState->mapLite, pGameState->currentRoom->id)) != NULL)
						fnListPrint(pGameState, pDynRoom->objs);
					/*******************************************************************************************/		

                    /*Obtain access to the dynamic data and call fnListPrint to print out descriptions*/
                    if((pDynRoom = fnTepDynRoomSearch(pGameState->mapLite, pGameState->currentRoom->id)) != NULL)
					{
	                    fnListPrint(pGameState, pDynRoom->people);//pGameState->currentRoom->objs);
					}
                }
                else
                {
                    printf("You cannot go that way.\n");
                    return FAILURE;
                }
            }/* end if((msg.Reserved & TEP_ROOM) != TEP_ROOM) */
            else if((msg.Reserved & TEP_ROOM) == TEP_ROOM)
            {
                TepIDUnNode *pRoom;

                if((pRoom = fnTepIDSearch(pGameState->currentRoom->objs, msg.Reserved)) == NULL)
                {
                    printf("You cannot go that way.\n");
                    return FAILURE;
                }
                if((newRoom = fnTepRoomSearch(pGameState->FullMap, pRoom->NODE)) != NULL)
				{
					/*Set the new room as the current room in the game state*/
                    pGameState->currentRoom = newRoom;
                    //fnPrintOut(NOPAUSE, NULL,NULL,1, pGameState->currentRoom->name);
                    fnPrintOut(NOPAUSE, KillerNumber,NULL,2, "\n", pGameState->currentRoom->desc);
					
					
#if 1
				/******************************** Added by Nate 4/12/01  ********************************/		
				/* PrintOut the murder if this is the right place */
				if(pGameState->currentRoom->id == pCurDeath->death->location)
					fnPrintOut(NOPAUSE | CharGenderFind(pCurDeath->victim), KillerNumber, pVictim->data,2, "\n", pCurDeath->death->desc);
				/*******************************************************************************/
#endif
					/** PrintOut all the clues in this room if this is the right place Added by nate,  4/7/01 **/
					if( (pDynRoom = fnTepDynRoomSearch(pGameState->mapLite, pGameState->currentRoom->id)) != NULL)
						fnListPrint(pGameState, pDynRoom->objs);
					/*******************************************************************************************/		

                    /*Obtain access to the dynamic data and call fnListPrint to print out descriptions*/
                    if((pDynRoom = fnTepDynRoomSearch(pGameState->mapLite, pGameState->currentRoom->id)) != NULL)
					{
	                    fnListPrint(pGameState, pDynRoom->people);//pGameState->currentRoom->objs);
					}
				}
            }/* end if((msg.Reserved & TEP_ROOM) == TEP_ROOM) */

        }/* end if( (msg.Command & COM_GO) == COM_GO)*/

        /*
        switch(msg.Command)
        {
        case COM_NORTH:
            newRoom = fnTepRoomSearch(pGameState->FullMap, room->rooms->NODE);
            break;
        case COM_EAST:
            newRoom = fnTepRoomSearch(pGameState->FullMap, room->rooms->NEXT->NODE);
            break;
        case COM_SOUTH:
            newRoom = fnTepRoomSearch(pGameState->FullMap, room->rooms->NEXT->NEXT->NODE);
            break;
        case COM_WEST:
            newRoom = fnTepRoomSearch(pGameState->FullMap, room->rooms->NEXT->NEXT->NEXT->NODE);
            break;
        case COM_GO:
            {
                TepIDUnNode *pRoom;

                if((pRoom = fnTepIDSearch(pGameState->currentRoom->objs, msg.Reserved)) == NULL)
                {
                    printf("You cannot go that way.\n");
                    return FAILURE;
                }
                newRoom = fnTepRoomSearch(pGameState->FullMap, pRoom->NODE);
                break;
            }
        } /*end switch*/

        /*Set the new room as the current room in the game state
        pGameState->currentRoom = newRoom;
        fnPrintOut(PAUSE, NULL,NULL,1, pGameState->currentRoom->desc);*/

        /*Obtain access to the dynamic data and call fnListPrint to print out descriptions
        pDynRoom = fnTepDynRoomSearch(pGameState->mapLite, pGameState->currentRoom->id);
        fnListPrint(pGameState, pGameState->currentRoom->objs);*/
    }/*end if*/
    else
    {
        printf("You cannot go that way.\n");
        return FAILURE;
    }

	/********** Added by nate on 3/29/01 **********/
	/* Fixes movement bug where even if you can't go somewhere you'd lose time, now with this you won't */
	pGameState->turnsLeft += ACT_GO;
	/**********************************************/

    return SUCCESS;
}/*end DisplayPlayerLocation Function*/
