#include "tep_incs.h"

/* Programmer Name: Nathaniel Cleveland
 * Date Created: Feb 20th, 2001
 * Program Updated by:          Jon Olson - Feb 25, 2001
 *
 * Program Description:         Loops through and fills a linked list of stuff.
 *
 * Program dependacies:         fnDefToHex, fnEditChunker
 * Takes:                       TepChar **ppListRoot,   the List Root which I'll link to
 *                              char pFileToOpen[80],       the file name i want to open
 * Gives:                       The BOOL value FAILURE (-1), if it doesn't work
 *                              BOOL value TRUE, if it does.
 */
BOOL fnTepEditCharacter(TepChar **ppListRoot, char pFileToOpen[80])
{
    TepChar     *pCurrNode = *ppListRoot;               /* point to the first item in the list*/
    TepChar     *pNewNode = NULL;                       /* pointer to the newly created node */
    FILE        *FileHandle = NULL;                     /* handle to the file I'm going to open */
    BOOL        bErrorCheck = TRUE;                     /* varible to hold bool results */
    char        cpString[ARRAYSIZE];                    /* string filled with file chunks */
    char        cpMemString[LONG_ARRAYSIZE];            /* used for long strings that aren't appreate for an array */

    memset(cpMemString, 0, LONG_ARRAYSIZE);             /* zero out the string(s) */
    memset(cpString, 0, ARRAYSIZE );

    /********************************************************************************/
    /**** 1. malloc mem for the list root, fail if we can't CALLER NOW DOES THIS ****/
    /********************************************************************************/

    /****************************************************************************/
    /**** 2. open the file, do a quick read to make sure the file is there ******/
    /****************************************************************************/
    if( (FileHandle  = fopen(pFileToOpen, "r" )) == NULL )
    {
      printf("Failed to open the file\n");
      return FAILURE;
    }

    /*********************************/
    /**** 3. fill in the stuct(s) ****/
    /*********************************/
    /* loop thru and fill the structs. Stop filling whenever we encounter the string END OF FILE */
    for( ; ; )
    {
        pNewNode = calloc(1, sizeof(TepChar));                  /* allocate memory for the new node */

        if (pCurrNode == NULL)
		    *ppListRoot = pCurrNode = pNewNode;

        if(pNewNode == NULL)                                        /* check for failure */
        {
            printf("failed to calloc memory\n");
            return FAILURE;
        }

        pNewNode->NEXT = NULL;                                      /* set the next node pointer to NULL */
        /* get the id from the file */
        if(fnEditChunker(cpString, FileHandle, '\t') == FALSE)      /* get the next chunk of info, check for END OF FILE */
        {
//            printf("break\n");
            break;
        }/* end if */

        /* fill in the id of the character, check for the use of #defs */
        if(atoi(cpString) == 0)                                     /* If this is a #def name, call fnDefToHex */
        {                                                           /* to convert it to its number. */
            pNewNode->id = fnDefToHex(cpString);
        }/* end if */
        else                                                        /* If it is just a number use that */
        {
            pNewNode->id = atoi(cpString);
        }/* end else */

//      printf("%d\n", pNewNode->id);                                   /* read what we got */
        memset(cpString, 0, strlen(cpString) );                     /* zero out the string */
        /* fill in the name memember */
        if(fnEditChunker(cpString, FileHandle, '\t') == FALSE)      /* get the next chunk of data, check for END OF FILE */
        {
            break;
        }

        /* fill in the name */
        pNewNode->name = calloc( strlen(cpString) + 1, sizeof(char));/* calloc mem for the string */

        if(pNewNode->name == NULL)                                  /* check for error in calloc */
        {
            printf("calloc didn't alloc mem for the name.\n");
            return FAILURE;
        }
                                                                    /* copy the string to the memory alloced string */
		strcpy(pNewNode->name, cpString);
//        pNewNode->name = memmove(pNewNode->name, cpString, strlen(cpString) );
//      printf("%s\n", pNewNode->name);

        /* fill in the desc memember */
        if(fnEditChunker(cpMemString, FileHandle, '\t') == FALSE)       /* get the next chunk of data, check for END OF FILE */
        {
            break;
        }

        /* fill in the desc */
        pNewNode->desc = calloc( strlen(cpMemString) + 1, sizeof(char));/* calloc mem for the string */

        if(pNewNode->desc == NULL)                                  /* check for error in calloc */
        {
            printf("calloc didn't alloc mem for the desc.\n");
            return FAILURE;
        }
                                                                    /* copy the string to the memory alloced string */
        pNewNode->desc = memcpy(pNewNode->desc, cpMemString, strlen(cpMemString) );
//      printf("%s\n", pNewNode->desc);


        /********fill in the locations member**********/

		for( ; ; )

		{
			if(fnEditChunker(cpString, FileHandle, ',') == FALSE)       /* get the next chunk of info, check for END OF FILE */
			{
				break;
			}/* end if */

			if(atoi(cpString) == 0)                                     /* If this is a #def name, call fnDefToHex */
			{                                                           /*  to convert it to its number. */
				TepID TEst = 0;
				TEst = fnDefToHex(cpString);
				//          pNewNode->NODE = fnDefToHex(cpString);
				fnTepIDAppend(&pNewNode->locations, TEst);
	//			printf("This was attached, 0x%X\n",TEst);
			}/* end if */

			if(getc(FileHandle) == '\n')                               /* pull out the next character in the file */
			{
				break;                                                  /* break if it matches the cComp */
			}/* end if */

			/* if not move the file pointer back by one */
			if(fseek(FileHandle, -1, SEEK_CUR ) != 0 )              /* check for failure to reset pointer */
			{
				printf("failed to reset the file pointer in fnTepEditUnNodeLink\n");
				return FALSE;
			}

		}/* end for */

		/* set character compliment counter */
		pNewNode->complimentCounter = SET_COMPLIMENT;
		if (rand() % 2)
		{
			pNewNode->complimentCounter += rand() % (ADJ_COMPLIMENT + 1);
		}
		else
		{
			pNewNode->complimentCounter -= rand() % (ADJ_COMPLIMENT + 1);
		}


        /* Test the node(s) */
        /*
        printf("0x%08X\n", pNewNode->id);
        printf("%s\n",pNewNode->name);
        printf("%s\n", pNewNode->desc);

        for (;;)
        {
			pTepIDUnNode pLoc = pNewNode->locations;
			if (pLoc != NULL && pLoc->NODE != 0)
			{
				printf(" - %08X\n", pLoc->NODE);
				pLoc = pLoc->NEXT;
			}
			else
			{
				printf(" - (NULL)\n");
				break;
			}
		}
		//getch();
		*/

        /* set this node to the list */
        pCurrNode->NEXT = pNewNode;
        //printf("pCurrNode = %08X\n", pCurrNode);
        //printf("pCurrNode->NEXT = %08X\n", pCurrNode->NEXT);

        /* point CurrNode at this node */
        pCurrNode = pNewNode;

    }/* end for */

    fflush(FileHandle);

    if( fclose(FileHandle) != 0)
    {
        printf("damn file wouldn't close");
    }

    return TRUE;
}/* End fnTepEditDeath */

/****************** DEBUG MAIN ******************/
/*
int main(void)
{
    char inputArray[255];
    BOOL bCheckBool;
    TepChar *pListRoot=NULL;

    pListRoot = calloc(1, sizeof(TepChar));

    while(1)
    {
        if(gets(inputArray) == NULL)
            break;

        if(inputArray == 0)
            break;
        bCheckBool = fnTepEditCharacter(&pListRoot, "CharacterContent.txt");

        if(bCheckBool == FAILURE || bCheckBool == FALSE)
        {
            printf("\nFailed\n");
        }
        else
        {
            printf("\nDidn't fail\n");
        }
    }
    free(pListRoot);
    return 0;
}
*/
/*
int main ( int argc, char *argv[], char *envp[])
{
    pTepChar pListRoot = NULL;

    if (argc > 1)
    {
        BOOL bCheckBool;

        pListRoot = calloc(1, sizeof(TepChar));
        bCheckBool = fnTepEditCharacter(&pListRoot, argv[1]);

        if(bCheckBool == FAILURE || bCheckBool == FALSE)
        {
            printf("\nFailed\n");
        }
        else
        {
            printf("\nDidn't fail\n");
        }
    }
    else
    {
        printf("Usage:\n  TepEditCharacter <filename> [outputfile]\n    Where <filename> is the name of the file to be processed.\n    Outputfile is the name of the file to write to.\n\n");
    }

    if (pListRoot != NULL)
    {
        FILE *ioStream = NULL;

        if (argc > 2)
        {
            ioStream = fopen(argv[2], "w" );
        }
        else
        {
            ioStream = fopen("tepCharacter.dat", "w");
        }

        if (ioStream != NULL)
        {
            fnTepEditWriteFile(TEP_CHARACTER, pListRoot->NEXT, ioStream);
            fclose(ioStream);
        }
        else
        {
            printf("There was an error in opening the output file.\n");
        }

        //fnTepCharDelete(&pListRoot, NULL);
    }

    return 0;
}
*/
