#include "TepEditDeath.h"

/* Programmer Name: Nathaniel Cleveland
 * Date Created: Feb 21st, 2001
 * Program Updated by:
 *
 * Program Description:			Loops through and fills a linked list of Deaths.
 *
 * Program dependacies:			fnEditChunker
 * Takes:						TepDeath **ppListRoot,	the List Root which I'll link to
 *								char pFileToOpen[80],	the file name I want to open
 * Gives:						The BOOL value FAILURE (-1), if it doesn't work
 *								BOOL value TRUE, if it does.
 */
BOOL fnTepEditDeath(TepDeath **ppListRoot, char pFileToOpen[80])
{
	TepDeath	*pCurrNode = *ppListRoot;				/* point to the first item in the list*/
    TepDeath	*pNewNode = NULL;						/* pointer to the newly created node */
	FILE		*FileHandle = NULL;						/* handle to the file I'm going to open */
	BOOL		bErrorCheck = TRUE;						/* varible to hold bool results */
	char		cpString[ARRAYSIZE];					/* string filled with file chunks */
	char		cpMemString[LONG_ARRAYSIZE];			/* used for long strings that aren't appreate for an array */

	memset(cpMemString, 0, LONG_ARRAYSIZE);				/* zero out the string(s) */
	memset(cpString, 0, ARRAYSIZE );

	/********************************************************************************/
	/**** 1. malloc mem for the list root, fail if we can't CALLER NOW DOES THIS ****/
	/********************************************************************************/

	/****************************************************************************/
	/**** 2. open the file, do a quick read to make sure the file is there ******/
	/****************************************************************************/
	if( (FileHandle  = fopen(pFileToOpen, "r" )) == NULL )
	{
      printf("Failed to open the file\n");
	  return FAILURE;
	}


	/*********************************/
	/**** 3. fill in the stuct(s) ****/
	/*********************************/
	/* loop thru and fill the structs. Stop filling whenever we encounter the string END OF FILE */
	for( ; ; )
	{

		pNewNode = calloc(1, sizeof(TepDeath));					/* allocate memory for the new node */

        if (pCurrNode == NULL)
		    *ppListRoot = pCurrNode = pNewNode;

		if(pNewNode == NULL)										/* check for failure */
		{
			printf("failed to calloc memory\n");
			return FAILURE;
		}

		pNewNode->NEXT = NULL;										/* set the next node pointer to NULL */

		if(fnEditChunker(cpString, FileHandle, '\t') == FALSE)		/* get the next chunk of info, check for END OF FILE */
		{
//			printf("break\n");
			break;
		}/* end if */

		/* fill in the id, check for the use of #defs */
		if(atoi(cpString) == 0)										/* make sure this string is a number */
		{
			printf("error in file, it wasn't written correctly\n");	/* return fail if it wasn't, and an error message */
			return FAILURE;
		}/* end if */
		else														/* If it is just a number use that */
		{
			pNewNode->id = TEP_DEATH | atoi(cpString);
		}/* end else */







		/* read in the location for the death */
		if(fnEditChunker(cpString, FileHandle, '\t') == FALSE)		/* get the next chunk of info, check for END OF FILE */
		{
			break;
		}
		//printf("EditChunker gives us ^ %s\n", cpString);
		/* fill in the location, check for the use of #defs */
		if(atoi(cpString) == 0)										/* make sure this string is a number */
		{
			pNewNode->location = fnDefToHex(cpString);				/* find the #def name given via deftohex */
		}/* end if */
//		else														/* If it is just a number use that */
//		{
//			pNewNode->location = TEP_DEATH + atoi(cpString);
//		}/* end else */

		memset(cpString, 0, strlen(cpString) );						/* zero the string */







		/*** Read in the description of the death ***/
		if(fnEditChunker(cpMemString, FileHandle, '\n') == FALSE)	/* get the next chunk of info, check for END OF FILE */
		{
			break;
		}

		/* fill in the desc */
		pNewNode->desc = calloc( strlen(cpMemString) + 1, sizeof(char));	/* calloc mem for the string */

		if(pNewNode->desc == NULL)											/* check for error in calloc */
		{
			printf("calloc didn't alloc mem for the desc.\n");
			return FAILURE;
		}
														 /* copy the string to the memory alloced string */
		pNewNode->desc = memmove(pNewNode->desc, cpMemString, strlen(cpMemString) );

		/* fill in the location, check for the use of #defs */
//		if(atoi(cpString) == 0) /* make sure this string is a number */
//		{
//			pNewNode->location = fnDefToHex(cpString); /* find the #def name given via deftohex */
//		}/* end if */
//		else /* If it is just a number use that */
//		{
//			pNewNode->location = TEP_DEATH + atoi(cpString);
//		}/* end else */

		/* Test the node(s) */

//		printf("0x%x\n", pNewNode->id);
		//printf("0x%x\n", pNewNode->location);
//		printf("%s\n", pNewNode->desc);


		/* set this node to the list */
		pCurrNode->NEXT = pNewNode;

		/* point CurrNode at this node */
		pCurrNode = pNewNode;
	}/* end for */

//printf("Exit Death\n");
	if( fclose(FileHandle) != 0)
	{
		printf("damn file wouldn't close");
	}

	return TRUE;
}/* End fnTepEditDeath */
/****************** DEBUG MAIN ******************/
/*
int main(void)
{
	char inputArray[255];
	BOOL bCheckBool;
	TepDeath *pListRoot = NULL;

	pListRoot = calloc(1, sizeof(TepChar));

	while(1)
	{
		if(gets(inputArray) == NULL)
			break;

		if(inputArray == 0)
			break;

		bCheckBool = fnTepEditDeath(&pListRoot, "DeathContent.txt");
		if(bCheckBool == FALSE)
		{
			printf("\nFailed\n");
		}
		else
		{
			printf("\nDidn't fail\n");
		}
	}
	free(pListRoot);
	return 0;
}
*/
/*
int main ( int argc, char *argv[], char *envp[])
{
    pTepDeath pListRoot = NULL;

    if (argc > 1)
    {
        BOOL bCheckBool;

        pListRoot = calloc(1, sizeof(TepDeath));
        bCheckBool = fnTepEditDeath(&pListRoot, argv[1]);

        if(bCheckBool == FAILURE || bCheckBool == FALSE)
        {
            printf("\nFailed\n");
        }
        else
        {
            printf("\nDidn't fail\n");
        }
    }
    else
    {
        printf("Usage:\n  TepEditDeath <filename> [outputfile]\n    Where <filename> is the name of the file to be processed.\n    Outputfile is the name of the file to write to.\n\n");
    }

    if (pListRoot != NULL)
    {
        FILE *ioStream = NULL;

        if (argc > 2)
        {
            ioStream = fopen(argv[2], "w" );
        }
        else
        {
            ioStream = fopen("tepDeath.dat", "w");
        }

        if (ioStream != NULL)
        {
            fnTepEditWriteFile(TEP_DEATH, pListRoot->NEXT, ioStream);
            fclose(ioStream);
        }
        else
        {
            printf("There was an error in opening the output file.\n");
        }

        //fnTepCharDelete(&pListRoot, NULL);
    }

    return 0;
}
*/
