/* Program ID:          tep_main.c
 * Programmer Name:     Matt Phillips
 * Date Created:        January 24, 2001
 * Program Updated by:	Jon Olson			Date of Update: 03/05/01
 *						Matt Phillips       Date of Update: 01/24/01
 * Program Description: tep_main.c handles the game's main execution loop.
 *                      This includes initialization, clean up, and all
 *                      menu selection handling.
 * Dependencies:        "tep_incs.h", "tep_main.h"
 * Input Files:         N/A
 * Output Files:        N/A
 */
#include <process.h>
#include <math.h>
#include "tep_incs.h"


/* Function:            fnCenterText
 * Parameters:          const char *
 * Return Value:        whether something was done or not
 * Side Effects:        prints data to screen
 * Description:         read side effects
 */
BOOL fnCenterText( const char *pszStr )
{
	if (pszStr != NULL)
	{
		int i;
		int len = (80 - strlen( pszStr )) / 2;

		for (i = 0; i < len; i++)
		{
			putchar(' ');
		}
		
		printf( pszStr );
		
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

 
 
/* Function:            main
 * Parameters:          void
 * Return Value:        0 (for now...)
 * Side Effects:        Well, this IS the game.
 * Description:         Checks for needed game files, initializes game data, and enters the menu
 *                      control loop. Before quitting cleans up all memory.
 */
int     main( void )
{
    TepCurState     GameState;              /* THE Game State data. Initialized by the call to fnInitPersistantStructs() below */
    BOOL    bContinue = TRUE;               /* flag is true until the user decides to quit */

    /* check for missing data files. If any are missing abort */
    if(fnCheckTepFiles() == TRUE)
    {
        /* try to load the game map and other data */
        if(fnInitPersistantStructs(&GameState) == TRUE)
        {
            /* main program loop. only exited when the user chooses to quit */
            while(bContinue)
            {
                fnTitleScreen();
                bContinue = fnControl(fnGetInputChar(), &GameState);
            }/* end while(bContinue) */
        }
		#if defined(_DEBUG)
		else
		{
			printf("debug::persistent failed.");
		}
		#endif

        fnFreePersistantStructs(&GameState);
    }
    #if defined(_DEBUG)
    else
    {
		printf("debug::check file failed.");
	}
	#endif
    return 0;
}



/* Function:            fnCheckTepFiles
 * Parameters:          void
 * Return Value:        TRUE or FALSE - for success or failure
 * Side Effects:        Outputs error message for each missing file.
 * Description:         Checks for needed game files. If any are missing it prints out which
 *                      files are missing and returns failure.
 */
BOOL    fnCheckTepFiles( void )
{
    BOOL fRet = TRUE;
    FILE *ioStream = NULL;

    if ((ioStream = fopen(IPS_CHARACTER,"rb")) != NULL && fRet)
    {
		#if defined(_DEBUG)
		printf("character\n");
		#endif
		fclose(ioStream);
	}
	else
	{
		#if defined(_DEBUG)
		printf("!character\n");
		#endif
		if (ioStream != NULL)
			fclose(ioStream);
		fRet = FALSE;
	}

    if ((ioStream = fopen(IPS_CLUE,"rb")) != NULL && fRet)
    {
		#if defined(_DEBUG)
		printf("clue\n");
		#endif
		fclose(ioStream);
	}
	else
	{
		#if defined(_DEBUG)
		printf("!clue\n");
		#endif
		if (ioStream != NULL)
			fclose(ioStream);
		fRet = FALSE;
	}

    if ((ioStream = fopen(IPS_COMMAND,"rb")) != NULL && fRet)
    {
		#if defined(_DEBUG)
		printf("command\n");
		#endif
		fclose(ioStream);
	}
	else
	{
		#if defined(_DEBUG)
		printf("!command\n");
		#endif
		if (ioStream != NULL)
			fclose(ioStream);
		fRet = FALSE;
	}

    if ((ioStream = fopen(IPS_DEATH,"rb")) != NULL && fRet)
    {
		#if defined(_DEBUG)
		printf("death\n");
		#endif
		fclose(ioStream);
	}
	else
	{
		#if defined(_DEBUG)
		printf("!death1\n");
		#endif
		if (ioStream != NULL)
			fclose(ioStream);
		fRet = FALSE;
	}

    if ((ioStream = fopen(IPS_TALK,"rb")) != NULL && fRet)
    {
		#if defined(_DEBUG)
		printf("talk\n");
		#endif
		fclose(ioStream);
	}
	else
	{
		#if defined(_DEBUG)
		printf("!talk\n");
		#endif
		if (ioStream != NULL)
			fclose(ioStream);
		fRet = FALSE;
	}

    if ((ioStream = fopen(IPS_ROOM,"rb")) != NULL && fRet)
    {
		#if defined(_DEBUG)
		printf("room\n");
		#endif
		fclose(ioStream);
	}
	else
	{
		#if defined(_DEBUG)
		printf("!room\n");
		#endif
		if (ioStream != NULL)
			fclose(ioStream);
		fRet = FALSE;
	}

    return fRet;
}



/* Function:            fnInitPersistantStructs
 * Parameters:          needs a pointer to TepCurState
 * Return Value:        TRUE (for now... should be success failure of database loading)
 * Side Effects:        loads game databases into memory and inits the GameState struct
 * Description:         Loads the game map. Loads dialogue. NULLs all pointers that wont
 *                      be used until a game is actually started.
 */
BOOL    fnInitPersistantStructs(TepCurState *pGameState)
{
    BOOL fRet = TRUE;
    void *lpBuf1 = NULL;

    /* pregame values and NULL pointers */
    pGameState->turnsLeft = 0;
    pGameState->fNeedSave = FALSE;
    pGameState->currentRoom = NULL;
    pGameState->killersList = NULL;
    pGameState->pastMurders = NULL;
    pGameState->liveList = NULL;
    pGameState->player = NULL;
    pGameState->currentMurder = NULL;
    pGameState->mapLite = NULL;
	pGameState->genConv = NULL;
	pGameState->keywords = NULL;
    pGameState->FullMap = NULL;

    /* load the game map */
    CreateRoomList(&pGameState->FullMap);

    /* load commands */
	fnTepEditCommand((TepDataString **)&lpBuf1, IPS_COMMAND);

	/* generate keywords */
	fRet = TepKeywordGenerator(pGameState, lpBuf1, NULL, NULL,  pGameState->FullMap);
	if( fRet == FAILURE || fRet == FALSE )
	{
		printf("Kewords failed to generate correctly.");
		return FAILURE;
	}

	/* print out the current keyword list
	{
		TepDataString *pData;

		printf("\nCurrent keywords\n");
		for(pData = pGameState->keywords; pData != NULL; pData = pData->NEXT)
		{
			printf("%s\t%x\n", pData->data, pData->id);
		}
		printf("\n");
	}*/

    return fRet;
}



/* Function:            fnTitleScreen
 * Parameters:          void
 * Return Value:        void
 * Side Effects:        prints out the title screen
 * Description:         Displays the title screen. Oooooooh.
 */
void    fnTitleScreen( void )
{
    /* Print Title Screen */
    printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
    fnCenterText("The Emissary of Tep\n");
    fnCenterText("A Monkey18 Production\n");

    /* Menu */
    printf("\n\n\n\n");
    fnCenterText("(N)ew Game      (L)oad Game      (Q)uit\n");
    //fnCenterText("(N)ew Game                       (Q)uit\n");
	//fnCenterText("(A)lpha         (Q)uit\n");
	printf("\n");
}



/* Function:            fnGetInputChar
 * Parameters:          void
 * Return Value:        A single uppercase char.
 * Side Effects:        Flushes stdin before grabbing a character
 * Description:         This gets the users input and returns it as an uppercase char.
 */
char    fnGetInputChar( void )
{
    char    cInput;         /* the input char */

    /* flush stdin and read in the players command*/
    fflush(stdin);

    /* Pause the game until the player whacks any key. */
    while(!_kbhit());

    /* grab the key that was pressed */
    cInput = getch();

    /* return the input - capitalized */
    return toupper(cInput);
}



/* Function:            fnControl
 * Parameters:          a char that specifies what action to take
 *                      a pointer to TepCurState
 * Return Value:        TRUE unless it fails. If it fails, what can we do? Just laugh.
 * Side Effects:        Frees all outstanding memory. (unloads the databases)
 * Description:         Cleans up memory before we exit the game. It's the ultimate destroyer if you're into that sort of thing.
 */
BOOL    fnControl(char cCommand, TepCurState *pGameState)
{
    switch(cCommand)
    {
        /* the user selected the quit game option */
        case 'Q':
            {
                /* make sure it's not an accident */
                printf("\nAre you sure you want to quit? (Y/N)");

                /* if the user inputs Y, return false */
                if(fnGetInputChar() == 'Y')
                {
                    return FALSE;
                }
                else return TRUE;
            }

		/* load and run a friggin game! */
		case 'L':
			{
				TepParse StartRoom;
				BOOL CheckRunGame = IA_END_OF_TURN;
				TepConvData *pWakeupMessage;

                /* run a new game */
				fnInitLoad( pGameState );

				/* load the game data */
				fnLoadGame( pGameState );

				/* set the first room */
				StartRoom.Command = COM_DISPLAY;
				StartRoom.Reserved = pGameState->currentRoom->id;

				/* display the player location */
				fnDisplayPlayerLocation( pGameState, StartRoom );

				/* run the turn */
				CheckRunGame = fnInterpretAction( pGameState );

                if(CheckRunGame == IA_END_OF_TURN)
				{
                    fnEndTurn( pGameState );
                    CheckRunGame = fnSacrificeTime( pGameState );
				}

				StartRoom.Command = COM_DISPLAY;
				StartRoom.Reserved = TEP_ROOM | ROOM_ROOM | 0x30;

				// loop - exit on quit or game over
				while(CheckRunGame != IA_USER_QUIT)
				{
					fnNewTurn( pGameState );

					/* wake up message */
					if( ( pWakeupMessage = fnTepConvDataSearchRand( pGameState->genConv, TEP_TALK | TALK_BODWAKEUP, 0 ) ) != NULL )
					{
						fnPrintOut( PAUSE, NULL, NULL, 1, pWakeupMessage->details );
					}
					/* sheroesha's daily hint */
					fnPrintOut(NOPAUSE, NULL,NULL,1,"\n\n");
					fnSheroeshaClue( pGameState );
					/* this should happen inside newturn */
					pGameState->currentRoom = fnTepRoomSearch( pGameState->FullMap, TEP_ROOM | ROOM_ROOM | 0x30 );
					fnDisplayPlayerLocation(pGameState, StartRoom);

					CheckRunGame = fnInterpretAction( pGameState );
					if(CheckRunGame == IA_END_OF_TURN)
					{
						fnEndTurn( pGameState );
						CheckRunGame = fnSacrificeTime( pGameState );
					}
				}// end of loop
                // clean up memory - dyn data, player data, reset gamestate
				fnEndGame( pGameState );
				
				/*** Print Out an end game screen ***/
				{
					char inputArray[80];
					/* Print Title Screen */
					printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
					fnCenterText("GAME OVER\n");
					fnCenterText("<press enter>\n");
					gets(inputArray);
					printf("\n\n\n\n\n");			
				}
			}
			break;

        /* the user selected the new game option */
        case 'N':
            {
				TepParse StartRoom;
				BOOL CheckRunGame = IA_END_OF_TURN;
				TepConvData *pWakeupMessage;

				StartRoom.Command = COM_DISPLAY;
				StartRoom.Reserved = TEP_ROOM | ROOM_ROOM | 0x30;

                /* run a new game */
                fnNewGame( pGameState );
                fnStoryIntroduction( pGameState );

                // loop - exit on quit or game over
				while(CheckRunGame != IA_USER_QUIT)
				{
                    fnNewTurn( pGameState );

					/* wake up message */
					if( ( pWakeupMessage = fnTepConvDataSearchRand( pGameState->genConv, TEP_TALK | TALK_BODWAKEUP, 0 ) ) != NULL )
					{
						fnPrintOut( PAUSE, NULL, NULL, 1, pWakeupMessage->details );
					}
					/* sheroesha's daily hint */
					fnPrintOut(NOPAUSE, NULL,NULL,1,"\n\n");
					fnSheroeshaClue( pGameState );
					/* this should happen inside newturn */
					pGameState->currentRoom = fnTepRoomSearch( pGameState->FullMap, TEP_ROOM | ROOM_ROOM | 0x30 );
					fnDisplayPlayerLocation(pGameState, StartRoom);

                    CheckRunGame = fnInterpretAction( pGameState );
                    if(CheckRunGame == IA_END_OF_TURN)
					{
                        fnEndTurn( pGameState );
                        CheckRunGame = fnSacrificeTime( pGameState );
					}
				}// end of loop
                // clean up memory - dyn data, player data, reset gamestate
				fnEndGame( pGameState );
				
				/*** Print Out an end game screen ***/
				{
					char inputArray[80];
					/* Print Title Screen */
					printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
					fnCenterText("GAME OVER\n");					
					fnCenterText("<press enter>\n");
					gets(inputArray);
					printf("\n\n\n\n\n");					
				}

            }
            break;
    }/* end switch(cCommand) */

    return TRUE;
}


/* init pointers for the loading of data */
BOOL fnInitLoad( TepCurState *pGameState )
{
    /* seed the random number table with the current time */
    srand((unsigned int)pow((double)time(NULL),(double)_getpid()));

    /* create the player struct !!! needs to be freed by the end game function !!! */
    pGameState->player = malloc( sizeof(TepPlayer) );

    /* init the player struct - everything is NULL for now */
    pGameState->player->name = NULL;
    pGameState->player->evidenceAgainst = NULL;
    pGameState->player->evidenceGeneric = NULL;
    pGameState->player->oldEvidenceAgainst = NULL;
    pGameState->player->oldEvidenceGeneric = NULL;

    /* the player should be asked to save if they quit the game later on */
    pGameState->fNeedSave = TRUE; // ha ha - yeah right

	return TRUE;
}

/* Function:            fnNewGame
 * Parameters:          needs a pointer to TepCurState
 * Return Value:        TRUE
 * Side Effects:        Initializes the TepCurState struct with new game values.
 *                      Allocates a Player struct and fills it's values.
 * Description:         Prepares data for a new game. Picks the 'mafia' Randomly sets 'trust'
 *                      values for characters.
 */
BOOL    fnNewGame(TepCurState *pGameState)
{
	TepDBConvData *pTepDBConv = NULL;	/* holds the conversation data as it is loaded but is deleted once it is attached to a character */
	void *pCharacterList = NULL;		/* holds the character list temporarily - just until it is passed to the liveList */
	BOOL fRet;							/* holds returns from the load functons */

    /* seed the random number table with the current time */
    srand((unsigned int)pow((double)time(NULL),(double)_getpid()));

    /* create the player struct !!! needs to be freed by the end game function !!! */
    pGameState->player = malloc( sizeof(TepPlayer) );

    /* init the player struct - everything is NULL for now */
    pGameState->player->name = NULL;
    pGameState->player->evidenceAgainst = NULL;
    pGameState->player->evidenceGeneric = NULL;
    pGameState->player->oldEvidenceAgainst = NULL;
    pGameState->player->oldEvidenceGeneric = NULL;

    /* the player should be asked to save if they quit the game later on */
    pGameState->fNeedSave = TRUE;



    /* fill the list of live characters with all 20 characters */
    pGameState->liveList = NULL;

	/* load characters *********/
    fRet = fnTepEditCharacter(&pGameState->liveList, IPS_CHARACTER);
	if (fRet != TRUE)
	{
		printf("Failed to load character content\n");
		return FALSE;
		//pGameState->liveList = (pTepChar)lpvMarker;
		//lpvMarker = NULL;
	}


	/* Load Talk/Conv ************/
	fRet = fnTepEditTalk(&pTepDBConv, IPS_CONVERSATION);
	if(fRet != TRUE)
	{
		printf("Failed to load talk content\n");
		return FALSE;
	}

	/* attach the conversations to the characters */
	fRet = fnAttachTalk(pTepDBConv, pGameState->liveList, &pGameState->genConv);

	/* free up the memory taken by pTepDBConv */
	{
		TepDBConvData *pTemp;

		while( pTepDBConv != NULL )
		{
			pTemp = pTepDBConv;
			pTepDBConv = pTepDBConv->NEXT;

			free( pTemp );
		}
	}

	/* generate character keywords */
	fRet = TepKeywordGenerator(pGameState, NULL, pGameState->liveList, NULL, NULL);

#if 0
	{
		TepConvData *pConv = pGameState->genConv;
		TepChar		*pChar = pGameState->liveList;

/*		printf("Printing GenericConversation Data:\n");
		while( pConv != NULL )
		{
			printf("ID: %8x\n", pConv->id);
			pConv = pConv->NEXT;
		}

		printf("Printing CharacterConversation Data:\n");
		while( pChar != NULL )*/
		{
			printf("\t%s's:", pChar->name);

			pConv = pChar->conv;

			while( pConv != NULL )
			{
				printf("\t\tID: %8x %20s\n", pConv->id, pConv->details);
				pConv = pConv->NEXT;
			}

			//pChar = pChar->NEXT;
		}
	}
#endif


//	printf("Running CharacterShuffle... \n");
    /* pick mafia, random character data, etc. */
    if( !fnCharacterShuffle(pGameState) )
	{
		/* out of memory */
		return FALSE;
	}


    return TRUE;
}



/* Function:            fnCharacterShuffle
 * Parameters:          needs a pointer to TepCurState
 * Return Value:        TRUE
 * Side Effects:        Points the pGameState->rametepList to a list of bad guys.
 *                      Generates the 'whack' list for every character.
 *                      Generates the trust value of the player. Inits the compliment counter.
 *                      Sets irrelevant pointers to NULL.
 * Description:         Read above. This is the character shuffler.
 */
BOOL    fnCharacterShuffle(TepCurState *pGameState)
{
    int i;
	int iLiveChars = 0;		/* the number of live characters at the beginning of the game */
	int iServantNum = 0;    /* counts the number of picked servants of rametep */
	int iRandom = 0;		/* holds a random number for character choosing and other stuff */
    TepChar *pCurrentChar = pGameState->liveList;

	/* count the number of live characters */
	while( pCurrentChar != NULL )
	{
		iLiveChars++;
		pCurrentChar = pCurrentChar->NEXT;
	}
//	printf("\nThere are %d characters to choose from.\n", iLiveChars);
	if( iLiveChars < 1 )
	{
		return FALSE;
	}

	/* choose the four servants of rametep */
	for( i = 0; i < BASE_SERVANTS; i++ )
	{
		fnTepIDAppend( &pGameState->killersList, (fnPickRandGoodChar( pGameState ))->id );
	}

	/* remove the other killers from each other's whacklists */
	{
		pTepIDUnNode pKillers = pGameState->killersList;
		pTepChar pBaddy = NULL;

		/* loop through each killer */
		while (pKillers != NULL)
		{
			pTepIDUnNode pKiller = pGameState->killersList;
			pTepWhack pWhack= fnTepCharSearch(pGameState->liveList, pKillers->NODE)->whacklist;

			/* remove the other killers from this killers list. */
			while (pKiller != NULL)
			{
				fnTepWhackDelete(&pWhack,fnTepWhackSearch(pWhack, pKiller->NODE));
				pKiller = pKiller->NEXT;
			}

			pKillers = pKillers->NEXT;
		}
	}

	/* set currentchar to the first char... */
	pCurrentChar = pGameState->liveList;

	/* randomly set the trust factor of all the characters */
	while( pCurrentChar != NULL )
	{
		TepChar *pWhackChar = NULL;		/* pointer to a TepChar used only in the generation of a character's whack list */

		if ((pCurrentChar->id & CHAR_SHEROESHA) != CHAR_SHEROESHA)
		{
			/* choose a random number for the trust factor between a low and high and set the character's trust*/
			iRandom = ( rand() % ( BASE_TRUST_HIGH - BASE_TRUST_LOW ) ) + BASE_TRUST_LOW;
			pCurrentChar->like = (unsigned char)iRandom;

			//printf("%s's trust factor towards the player is %d.\n", pCurrentChar->name, pCurrentChar->like);
			//printf("%s's trust factor towards other characters is:\n", pCurrentChar->name);

			/* create the whack list - currently no consideration between servants... */
			/* init pWhackChar */
			pWhackChar = pGameState->liveList;

			#if defined(_DEBUG)
			printf("WhackList Creation for %s\n", pCurrentChar->name);
			#endif

			/* loop through every character, add them to the whacklist and set the whack value */
			while( pWhackChar != NULL )
			{
				/* pick a random whack value */
				iRandom = rand() % WHACK_VAL_LIMIT;

				/* add the current character to the whack list - init the whack value */
				if (((pWhackChar->id & CHAR_SHEROESHA) != CHAR_SHEROESHA) && (pWhackChar->id != pCurrentChar->id))
				{
					if( fnTepWhackAppend( &pCurrentChar->whacklist, pWhackChar->id, (USHORT)iRandom ) == FAILURE )
					{
						/* out of memory */
						return FALSE;
					}
					#if defined(_DEBUG)
					else
					{
						printf("    %s\n", pWhackChar->name);
					}
					#endif
				}

				//printf("\t%10s  %d\n", pWhackChar->name, iRandom);

				/* increment to the next character */
				pWhackChar = pWhackChar->NEXT;
			}/* end while( pWhackChar != NULL ) */
		}

		/* go to the next character */
		pCurrentChar =  pCurrentChar->NEXT;
	}

    return TRUE;
}


BOOL fnEndGame( TepCurState *pGameState )
{
	/* clean up generic conversations */
	fnTepConvDataDelete( &pGameState->genConv, NULL );
	pGameState->genConv = NULL;

	/* clean up dynamic keywords - all but commands and rooms */
	{
		TepDataString **pListRoot = &pGameState->keywords;
		while ( pListRoot != NULL  &&  *pListRoot != NULL  &&  ((*pListRoot)->id & TEP_ROOM  ||  (*pListRoot)->id & TEP_COMMAND) )
		{
			pListRoot = &(*pListRoot)->NEXT;
		}
		fnTepDataStringDelete( pListRoot, NULL );
		*pListRoot = NULL;
	}

	/* clean up dynamic room data */
	fnTepDynRoomDelete( &pGameState->mapLite, NULL );
	pGameState->mapLite = NULL;

	/* clean up the past murders list */
	fnTepCurMurderDelete( &pGameState->currentMurder, NULL );
	pGameState->currentMurder = NULL;

	/* clean up the past murders list */
	fnTepCurMurderDelete( &pGameState->pastMurders, NULL );
	pGameState->pastMurders = NULL;

	/* clean up the evidence collected */
	fnTepEvidenceDelete( &pGameState->player->evidenceAgainst, NULL );
	pGameState->player->evidenceAgainst = NULL;
	fnTepEvidenceDelete( &pGameState->player->evidenceGeneric, NULL );
	pGameState->player->evidenceGeneric = NULL;
	fnTepEvidenceDelete( &pGameState->player->oldEvidenceAgainst, NULL );
	pGameState->player->oldEvidenceAgainst = NULL;
	fnTepEvidenceDelete( &pGameState->player->oldEvidenceGeneric, NULL );
	pGameState->player->oldEvidenceGeneric = NULL;

	/* free the player name */
	if (pGameState->player->name != NULL)
		free( pGameState->player->name );
	pGameState->player->name = NULL;

    /* free the player struct */
	if (pGameState->player != NULL)
    	free( pGameState->player );
    pGameState->player = NULL;

	/* clean up all living characters */
	fnTepCharDelete( &pGameState->liveList, NULL );
	pGameState->liveList = NULL;

	/* clean up the killers list */
	fnTepIDDelete( &pGameState->killersList, NULL );
	pGameState->killersList = NULL;
	return TRUE;
}


/* Function:            fnFreePersistantStructs
 * Parameters:          needs a pointer to TepCurState
 * Return Value:        TRUE unless it fails. If it fails, what can we do? Just laugh.
 * Side Effects:        Frees all outstanding memory. (unloads the databases)
 * Description:         Cleans up memory before we exit the game. It's the ultimate destroyer if you're into that sort of thing.
 */
BOOL    fnFreePersistantStructs(TepCurState *pGameState)
{
    /* delete the entire map */
    fnTepRoomDelete( &pGameState->FullMap, NULL );
    pGameState->FullMap = NULL;
    pGameState->currentRoom = NULL;

	/* delete the keyword list */
	fnTepDataStringDelete( &pGameState->keywords, NULL );
	pGameState->keywords = NULL;

    return 0;
}
