/* Programmer Name: Jon Olson
 * Date Created: Feb 27th, 2001
 * Program Updated by:
 *
 * Program Description:         A collection of functions for loading the database(s) into memory.
 *
 * Program Dependacies:         tep_fileio.h
 */

#include "tep_incs.h"
//#ifdef _DEBUG
//#undef _DEBUG
//#define _DEBUGDEFINED
//#endif


/*FUNCTION:     fnTepLD_EOF
 *PARAMETERS:   FILE *ioStream              the file you want to check
 *RETURN VALUE: BOOL TRUE					successful
 *              BOOL FALSE             		failure
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for reading from a database file
*/
BOOL fnTepLD_EOF(FILE *ioStream)
{
	DWORD dwPos = ftell(ioStream),
		dwSize = fseek(ioStream, 0, SEEK_END);

	fseek(ioStream, dwPos, SEEK_SET);

	return dwSize < dwPos;
}


/*FUNCTION:     fnTepLD_ReadArray
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              size_t cbSize          		count the bytes in objects being read.
 *RETURN VALUE: void *lpBuf					successful
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for reading from a database file
*/
void *fnTepLD_ReadArray(FILE *ioStream, void *lpBuf, size_t cbSize)
{
	DWORD x = 0;

	if (cbSize == 0)
		cbSize = 1;

#if defined(_DEBUG)
printf("fnTepLD_ReadArray=-=-=--==-=-=-\n");
#endif
	for(;;)
	{
		if (lpBuf == NULL)
		{
#if defined(_DEBUG)
printf("lpBuf == NULL\n");
#endif
			lpBuf = malloc(cbSize);
		}
		fread(&((char *)lpBuf)[x * cbSize], cbSize, 1, ioStream);
#if defined(_DEBUG)
if (cbSize == 1)
	printf("l = %08X\ts = %ld\tL = %08X\tc = %02X\tp = %08X\n", lpBuf, _msize(lpBuf), ((char *)lpBuf)[x], cbSize, ftell(ioStream));
else
	printf("l = %08X\ts = %ld\tL = %08X\tc = %02X\tp = %08X\n", lpBuf, _msize(lpBuf), ((DWORD *)lpBuf)[x], cbSize, ftell(ioStream));
//if (x == 10)
//exit(0);
#endif

		if (((char *)lpBuf)[x++ * cbSize] == 0)
		{
/*			if (lpBuf != NULL)
				free(lpBuf);
*/			break;
		}
		lpBuf = realloc(lpBuf, x * cbSize);
	}

#if defined(_DEBUG)
if (cbSize == 1)
printf("string = '%s'\n", (char *)lpBuf);
else
printf("data = %08X\n", ((DWORD *)lpBuf)[0]);
#endif

	return lpBuf;
}

/*FUNCTION:     fnTepLD_Read
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf		            successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for reading from a database file
*/
void *fnTepLD_Read(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
	BOOL fRet = FALSE;
	DWORD fFull = dwFlags & LD_FULL;
	dwFlags &= ~LD_FULL;

#if defined(_DEBUG)
printf("\n\nfnTepLD_Read-=-=-=-=--==\n");
#endif
	if (ioStream != NULL)
	{
		void *lpBuffer = lpBuf;
#if defined(_DEBUG)
printf("ioStream != NULL\n");
#endif
		{
			if (lpBuf == NULL)
			{
#if defined(_DEBUG)
printf("lpBuf == NULL\n");
#endif
				switch(dwFlags)
				{
				case TEP_CHARACTER:
					{
#if defined(_DEBUG)
printf("dwFlags was TEP_CHARACTER\n");
#endif
						if (dwFlags & TEP_CHARACTER)
						{
							pTepID pLoc = NULL;
							pTepIDUnNode pLoca, pLocal;
							long x = 0;

							pLoca = pLocal = NULL;

							lpBuf = calloc(sizeof(TepChar), 1);

							fread(&((pTepChar)lpBuf)->id, sizeof(((pTepChar)lpBuf)->id), 1, ioStream);
#if defined(_DEBUG)
printf("id\n");
#endif

							((pTepChar)lpBuf)->name = fnTepLD_ReadArray(ioStream, ((pTepChar)lpBuf)->name, sizeof(((pTepChar)lpBuf)->name[0]));
#if defined(_DEBUG)
printf("name, %08X\n", &((pTepChar)lpBuf)->name[0]);
printf("name=\"%s\"\n", ((pTepChar)lpBuf)->name);
#endif

							((pTepChar)lpBuf)->desc = fnTepLD_ReadArray(ioStream, ((pTepChar)lpBuf)->desc, sizeof(((pTepChar)lpBuf)->desc[0]));
#if defined(_DEBUG)
printf("desc, %08X\n", &((pTepChar)lpBuf)->desc[0]);
printf("desc=\"%s\"\n", ((pTepChar)lpBuf)->desc);
#endif

							pLoc = fnTepLD_ReadArray(ioStream, pLoc, sizeof(TepID));
#if defined(_DEBUG)
printf("locations, %08X\n", pLoc);
#endif

							x = 0;
							while (pLoc[x] != 0)
							{
#if defined(_DEBUG)
printf("while start\nlpbuf = %08X\n", lpBuf);
#endif
#if defined(_DEBUG)
printf("while start\npLoca = %08X\n", pLoca = malloc(sizeof(((pTepChar)lpBuf)->locations)));
#else
								pLoca = calloc(sizeof(((pTepChar)lpBuf)->locations), 1);
#endif
								if (pLoca != NULL)
								{
#if defined(_DEBUG)
printf("pLoca != NULL\n");
#endif
									pLoca->NODE = pLoc[x];
								}
								else
								{
#if defined(_DEBUG)
printf("pLoca == NULL\n");
#endif
								}
#if defined(_DEBUG)
printf("pLoca = %08X\n", pLoca);
printf("pLoca->NODE = %08X\n", pLoca->NODE);
#endif

								if (lpBuf == NULL)
								{
#if defined(_DEBUG)
printf("lpBuf == NULL\n");
#endif
								}
								else
								{
#if defined(_DEBUG)
printf("lpBuf != NULL\n");
#endif
								}

								if (((pTepChar)lpBuf)->locations == NULL)
								{
#if defined(_DEBUG)
printf("lpBuf->locations == NULL\n");
#endif
									((pTepChar)lpBuf)->locations = pLoca;
								}

								if (pLocal != NULL)
								{
#if defined(_DEBUG)
printf("pLocal != NULL\n");
#endif
									pLocal->NEXT = pLoca;
								}

								pLocal = pLoca;
								x++;
#if defined(_DEBUG)
printf("x = %d\n",x);
#endif
#if defined(_DEBUG)
printf("while end\n");
#endif
							}

							#if defined(_DEBUG)
							{
								printf("fnTepLD_Read---------------\n");
								printf("    ID:   %08X\n", ((pTepChar)lpBuf)->id);
								printf("    Name: \"%s\"\n", ((pTepChar)lpBuf)->name);
								printf("    Desc: \"%s\"\n", ((pTepChar)lpBuf)->desc);
								printf("    Locations:\n");

								if (((pTepChar)lpBuf)->locations != NULL)
								{
									pTepIDUnNode lpLoc = ((pTepChar)lpBuf)->locations;
									while (lpLoc != NULL)
									{
										printf("      -   %08X\n", lpLoc->NODE);
										lpLoc = lpLoc->NEXT;
									}
								}
								else
								{
									printf("      -   (NULL)\n");
								}
							}
							printf("here %d, %08X\n", __LINE__, lpBuf);
							if (fRet == FAILURE)
								printf("fret = failure\n");
							#endif;
						}

						break;
					}

				case TEP_CLUE:
				case TEP_TALK:
					{
#if defined(_DEBUG)
printf("dwFlags was TEP_TALK or TEP_CLUE\n");
#endif
						if (dwFlags & TEP_TALK || dwFlags & TEP_CLUE)
						{
							long x = 0;

							lpBuf = calloc(sizeof(TepConvData), 1);

							fread(&((pTepConvData)lpBuf)->id, sizeof(((pTepConvData)lpBuf)->id), 1, ioStream);
#if defined(_DEBUG)
printf("id\n");
#endif

							fread(&((pTepConvData)lpBuf)->attachedto, sizeof(((pTepConvData)lpBuf)->attachedto), 1, ioStream);
#if defined(_DEBUG)
printf("attachedto\n");
#endif

							((pTepConvData)lpBuf)->details = fnTepLD_ReadArray(ioStream, ((pTepConvData)lpBuf)->details, sizeof(((pTepConvData)lpBuf)->details[0]));
#if defined(_DEBUG)
printf("desc, %08X\n", &((pTepConvData)lpBuf)->details[0]);
printf("desc=\"%s\"\n", ((pTepConvData)lpBuf)->details);
#endif

							#if defined(_DEBUG)
							{
								printf("lpBuf = %08X\n", ((pTepConvData)lpBuf));
								printf("\tID:%08X\n", ((pTepConvData)lpBuf)->id);
								printf("\tAttached To:\"%s\"\n", ((pTepConvData)lpBuf)->attachedto);
								printf("\tDetails:\"%s\"\n", ((pTepConvData)lpBuf)->details);
							}
							printf("here %d, %08X\n", __LINE__, ((pTepConvData)lpBuf));
							if (fRet == FAILURE)
								printf("fret = failure\n");
							#endif;
						}

						break;
					}

				case TEP_DEATH:
					{
#if defined(_DEBUG)
printf("dwFlags was TEP_DEATH\n");
#endif
						if (dwFlags & TEP_DEATH)
						{
							pTepID pDea = NULL;
							pTepConvData pDeath, pDeath2;
							long x = 0;

							pDeath = pDeath2 = NULL;

							lpBuf = calloc(sizeof(TepDeath), 1);

							fread(&((pTepDeath)lpBuf)->id, sizeof(((pTepDeath)lpBuf)->id), 1, ioStream);
#if defined(_DEBUG)
printf("id\n");
#endif

							((pTepDeath)lpBuf)->desc = fnTepLD_ReadArray(ioStream, ((pTepDeath)lpBuf)->desc, sizeof(((pTepDeath)lpBuf)->desc[0]));
#if defined(_DEBUG)
printf("desc, %08X\n", &((pTepDeath)lpBuf)->desc[0]);
printf("desc=\"%s\"\n", ((pTepDeath)lpBuf)->desc);
#endif

							pDea = fnTepLD_ReadArray(ioStream, pDea, sizeof(TepID));
#if defined(_DEBUG)
printf("conv, %08X\n", pDea);
#endif

							x = 0;
							while (pDea[x] != 0)
							{
#if defined(_DEBUG)
printf("while start\nlpbuf = %08X\n", lpBuf);
#endif
#if defined(_DEBUG)
printf("while start\npDeath = %08X\n", pDeath = malloc(sizeof(((pTepDeath)lpBuf)->conv)));
#else
								pDeath = calloc(sizeof(((pTepDeath)lpBuf)->conv), 1);
#endif
								if (pDeath != NULL)
								{
#if defined(_DEBUG)
printf("pDeath != NULL\n");
#endif
									pDeath->id = pDea[x];
								}
								else
								{
#if defined(_DEBUG)
printf("pDeath == NULL\n");
#endif
								}
#if defined(_DEBUG)
printf("pDeath = %08X\n", pDeath);
printf("pDeath->id = %08X\n", pDeath->id);
#endif

								if (lpBuf == NULL)
								{
#if defined(_DEBUG)
printf("lpBuf == NULL\n");
#endif
								}
								else
								{
#if defined(_DEBUG)
printf("lpBuf != NULL\n");
#endif
								}

								if (((pTepDeath)lpBuf)->conv == NULL)
								{
#if defined(_DEBUG)
printf("lpBuf->conv == NULL\n");
#endif
									((pTepDeath)lpBuf)->conv = pDeath;
								}

								if (pDeath2 != NULL)
								{
#if defined(_DEBUG)
printf("pDeath2 != NULL\n");
#endif
									pDeath2->NEXT = pDeath;
								}

								pDeath2 = pDeath;
								x++;
#if defined(_DEBUG)
printf("x = %d\n",x);
#endif
#if defined(_DEBUG)
printf("while end\n");
#endif
							}
							pDeath2 = pDeath = NULL;



							pDea = fnTepLD_ReadArray(ioStream, pDea, sizeof(TepID));
#if defined(_DEBUG)
printf("clues, %08X\n", pDea);
#endif

							x = 0;
							while (pDea[x] != 0)
							{
#if defined(_DEBUG)
printf("while start\nlpbuf = %08X\n", lpBuf);
#endif
#if defined(_DEBUG)
printf("while start\npDeath = %08X\n", pDeath = malloc(sizeof(((pTepDeath)lpBuf)->clues)));
#else
								pDeath = calloc(sizeof(((pTepDeath)lpBuf)->clues), 1);
#endif
								if (pDeath != NULL)
								{
#if defined(_DEBUG)
printf("pDeath != NULL\n");
#endif
									pDeath->id = pDea[x];
								}
								else
								{
#if defined(_DEBUG)
printf("pDeath == NULL\n");
#endif
								}
#if defined(_DEBUG)
printf("pDeath = %08X\n", pDeath);
printf("pDeath->id = %08X\n", pDeath->id);
#endif

								if (lpBuf == NULL)
								{
#if defined(_DEBUG)
printf("lpBuf == NULL\n");
#endif
								}
								else
								{
#if defined(_DEBUG)
printf("lpBuf != NULL\n");
#endif
								}

								if (((pTepDeath)lpBuf)->clues == NULL)
								{
#if defined(_DEBUG)
printf("lpBuf->locations == NULL\n");
#endif
									((pTepDeath)lpBuf)->clues = (pTepClueData)pDeath;
								}

								if (pDeath2 != NULL)
								{
#if defined(_DEBUG)
printf("pDeath2 != NULL\n");
#endif
									pDeath2->NEXT = pDeath;
								}

								pDeath2 = pDeath;
								x++;
#if defined(_DEBUG)
printf("x = %d\n",x);
#endif
#if defined(_DEBUG)
printf("while end\n");
#endif
							}
							pDeath2 = pDeath = NULL;


							#if defined(_DEBUG)
							{
								printf("lpBuf = %08X\n", ((pTepDeath)lpBuf));
								printf("\tID:%08X\n", ((pTepDeath)lpBuf)->id);
								printf("\tDescription:\"%s\"\n", ((pTepDeath)lpBuf)->desc);
								printf("\tConversation Data:\n");
								if (((pTepDeath)lpBuf)->conv != NULL)
								{
									pTepConvData lpLoc = ((pTepDeath)lpBuf)->conv;
									while (lpLoc != NULL)
									{
										printf("\t - %08X\n", lpLoc->id);
										lpLoc = lpLoc->NEXT;
									}
								}
								else
								{
									printf("\t - (NULL)\n");
								}
								printf("\tClue Data:\n");
								if (((pTepDeath)lpBuf)->clues != NULL)
								{
									pTepClueData lpLoc = ((pTepDeath)lpBuf)->clues;
									while (lpLoc != NULL)
									{
										printf("\t - %08X\n", lpLoc->id);
										lpLoc = lpLoc->NEXT;
									}
								}
								else
								{
									printf("\t - (NULL)\n");
								}
							}
							printf("here %d, %08X\n", __LINE__, ((pTepDeath)lpBuf));
							if (fRet == FAILURE)
								printf("fret = failure\n");
							#endif;
						}

						break;
					}

				case TEP_COMMAND:
				case LD_STRINGS:
					{
#if defined(_DEBUG)
printf("dwFlags was TEP_COMMAND or LD_STRINGS\n");
#endif
						if (dwFlags & TEP_COMMAND || dwFlags & LD_STRINGS)
						{
							long x = 0;

							lpBuf = calloc(sizeof(TepDataString), 1);

							fread(&((pTepDataString)lpBuf)->id, sizeof(((pTepDataString)lpBuf)->id), 1, ioStream);
#if defined(_DEBUG)
printf("id\n");
#endif

							((pTepDataString)lpBuf)->data = fnTepLD_ReadArray(ioStream, ((pTepDataString)lpBuf)->data, sizeof(((pTepDataString)lpBuf)->data[0]));
#if defined(_DEBUG)
printf("desc, %08X\n", &((pTepDataString)lpBuf)->data[0]);
printf("desc=\"%s\"\n", ((pTepDataString)lpBuf)->data);
#endif

							#if defined(_DEBUG)
							{
								printf("lpBuf = %08X\n", ((pTepDataString)lpBuf));
								printf("\tID:%08X\n", ((pTepDataString)lpBuf)->id);
								printf("\tDetails:\"%s\"\n", ((pTepDataString)lpBuf)->data);
							}
							printf("here %d, %08X\n", __LINE__, ((pTepDataString)lpBuf));
							if (fRet == FAILURE)
								printf("fret = failure\n");
							#endif;
						}

						break;
					}

				default:
					{
						#if defined(_DEBUG)
						{
							printf("[fnTepLD_Read]::unknown or not supported value in flags, %08X\n",dwFlags);
						}
						#endif

						break;
					}
				}
			}
		}
	}
	else
	{
		fRet = FAILURE;
	}

	if (fRet == FAILURE && lpBuf != NULL)
	{
		//printf("fret = failure2\n");
		free(lpBuf);
		lpBuf = NULL;
	}
	else
	{
		fRet = SUCCESS;
	}

	//printf("fret = %d\nlpBuf = %08X\n", fRet, lpBuf);

	return lpBuf;
}



/*FUNCTION:     fnTepLD_Character
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf             	successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for parsing data from character file(s)
*/
void *fnTepLD_Character(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
	BOOL fRet = FAILURE;

#if defined(_DEBUG)
printf("\n\nfnTepLD_Character-----------------------------\n");
#endif
	printf("ioStream = %08X\n", ioStream);
	if (ioStream != NULL)
	{
#if defined(_DEBUG)
printf("ioStream != NULL, line:%d\n", __LINE__);
#endif
		lpBuf = fnTepLD_Read(ioStream, dwFlags, lpBuf);
	{
		pTepChar lpBuff = (pTepChar)lpBuf;
		if (lpBuff != NULL)
		{
			printf("lpBuff = %08X\n", lpBuff);
			printf("\tID:%08X\n", lpBuff->id);
			printf("\tName:\"%08X\"\n", lpBuff->name);
			printf("\tDescription:\"%08X\"\n", lpBuff->desc);
			printf("\tLocations:\n");
			if (lpBuff->locations != NULL)
			{
				pTepIDUnNode lpLoc = lpBuff->locations;
				while (lpLoc != NULL)
				{
					printf("\t - %08X, %08X\n", lpLoc->NODE, lpLoc->NEXT);
					lpLoc = lpLoc->NEXT;
				}
			}
			else
			{
				printf("\t - (NULL)\n");
			}
		}
		else
			printf("lpBuff = NULL\n");
	}
		if (lpBuf != NULL && dwFlags & LD_FULL && !fnTepLD_EOF(ioStream))
		{
			((pTepChar)lpBuf)->NEXT = fnTepLD_Read(ioStream, dwFlags, lpBuf);
			printf("!EOF and LD_FULL hit!\n");
		}
		else
			printf("EOF or !LD_FULL hit\n");
#if defined(_DEBUGGER)
	{
		pTepChar lpBuff = (pTepChar)lpBuf;
		if (lpBuff != NULL)
		{
			printf("lpBuff = %08X\n", lpBuff);
			printf("\tID:%08X\n", lpBuff->id);
			printf("\tName:\"%s\"\n", lpBuff->name);
			printf("\tDescription:\"%s\"\n", lpBuff->desc);
			printf("\tLocations:\n");
			if (lpBuff->locations != NULL)
			{
				pTepIDUnNode lpLoc = lpBuff->locations;
				while (lpLoc != NULL)
				{
					printf("\t - %08X\n", lpLoc->NODE);
					lpLoc = lpLoc->NEXT;
				}
			}
			else
			{
				printf("\t - (NULL)\n");
			}
		}
		else
			printf("lpBuff = NULL\n");
	}
#endif
	}

	return lpBuf;
}



/*FUNCTION:     fnTepLD_Location
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf             	successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for parsing data from character file(s)
*/
void *fnTepLD_Location(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
	BOOL fRet = FAILURE;

#if defined(_DEBUG)
printf("\n\nfnTepLD_Location-----------------------------\n");
#endif
	if (ioStream != NULL)
	{
#if defined(_DEBUG)
printf("ioStream != NULL\n");
#endif
		lpBuf = fnTepLD_Read(ioStream, dwFlags, lpBuf);
		if (lpBuf != NULL && dwFlags & LD_FULL && !fnTepLD_EOF(ioStream))
			((pTepRoom)lpBuf)->NEXT = fnTepLD_Read(ioStream, dwFlags, lpBuf);
#if defined(_DEBUG)
	{
		pTepRoom lpBuff = (pTepRoom)lpBuf;
		if (lpBuff != NULL)
		{
			printf("lpBuff = %08X\n", lpBuff);
			printf("\tID:%08X\n", lpBuff->id);
			printf("\tName:\"%s\"\n", lpBuff->name);
			printf("\tDescription:\"%s\"\n", lpBuff->desc);
			printf("\tCommands:%08X\n", lpBuff->cmd);
			printf("\tRooms:\n");
			if (lpBuff->rooms != NULL)
			{
				pTepIDUnNode lpLoc = lpBuff->rooms;
				while (lpLoc != NULL)
				{
					printf("\t - %08X\n", lpLoc->NODE);
					lpLoc = lpLoc->NEXT;
				}
			}
			else
			{
				printf("\t - (NULL)\n");
			}
			printf("\tObjects:\n");
			if (lpBuff->objs != NULL)
			{
				pTepIDUnNode lpLoc = lpBuff->objs;
				while (lpLoc != NULL)
				{
					printf("\t - %08X\n", lpLoc->NODE);
					lpLoc = lpLoc->NEXT;
				}
			}
			else
			{
				printf("\t - (NULL)\n");
			}
		}
		else
			printf("lpBuff = NULL\n");
	}
#endif
	}

	return lpBuf;
}



/*FUNCTION:     fnTepLD_Conversation
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf             	successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for parsing data from character file(s)
*/
void *fnTepLD_Conversation(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
	BOOL fRet = FAILURE;

#if defined(_DEBUG)
printf("\n\nfnTepLD_Conversation-----------------------------\n");
#endif
	if (ioStream != NULL)
	{
#if defined(_DEBUG)
printf("ioStream != NULL\n");
#endif
		lpBuf = fnTepLD_Read(ioStream, dwFlags, lpBuf);
		if (lpBuf != NULL && dwFlags & LD_FULL && !fnTepLD_EOF(ioStream))
			((pTepConvData)lpBuf)->NEXT = fnTepLD_Read(ioStream, dwFlags, lpBuf);
#if defined(_DEBUG)
	{
		pTepConvData lpBuff = (pTepConvData)lpBuf;
		if (lpBuff != NULL)
		{
			printf("lpBuff = %08X\n", lpBuff);
			printf("\tID:%08X\n", lpBuff->id);
			printf("\tAttached To:\"%s\"\n", lpBuff->attachedto);
			printf("\tDetails:\"%s\"\n", lpBuff->details);
		}
		else
			printf("lpBuff = NULL\n");
	}
#endif
	}

	return lpBuf;
}



/*FUNCTION:     fnTepLD_Death
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf             	successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for parsing data from character file(s)
*/
void *fnTepLD_Death(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
	BOOL fRet = FAILURE;

#if defined(_DEBUG)
printf("\n\nfnTepLD_Death-----------------------------\n");
#endif
	if (ioStream != NULL)
	{
#if defined(_DEBUG)
printf("ioStream != NULL\n");
#endif
		lpBuf = fnTepLD_Read(ioStream, dwFlags, lpBuf);
		if (lpBuf != NULL && dwFlags & LD_FULL && !fnTepLD_EOF(ioStream))
			((pTepDeath)lpBuf)->NEXT = fnTepLD_Read(ioStream, dwFlags, lpBuf);
#if defined(_DEBUG)
	{
		pTepDeath lpBuff = (pTepDeath)lpBuf;
		if (lpBuff != NULL)
		{
			printf("lpBuff = %08X\n", lpBuff);
			printf("\tID:%08X\n", lpBuff->id);
			printf("\tDescription:\"%s\"\n", lpBuff->desc);
			printf("\tConversation Data:\n");
			if (lpBuff->conv != NULL)
			{
				pTepConvData lpLoc = lpBuff->conv;
				while (lpLoc != NULL)
				{
					printf("\t - %08X\n", lpLoc->id);
					lpLoc = lpLoc->NEXT;
				}
			}
			else
			{
				printf("\t - (NULL)\n");
			}
			printf("\tClue Data:\n");
			if (lpBuff->clues != NULL)
			{
				pTepClueData lpLoc = lpBuff->clues;
				while (lpLoc != NULL)
				{
					printf("\t - %08X\n", lpLoc->id);
					lpLoc = lpLoc->NEXT;
				}
			}
			else
			{
				printf("\t - (NULL)\n");
			}
		}
		else
			printf("lpBuff = NULL\n");
	}
#endif
	}

	return lpBuf;
}



/*FUNCTION:     fnTepLD_Strings
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf             	successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for parsing data from character file(s)
*/
void *fnTepLD_Strings(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
	BOOL fRet = FAILURE;

#if defined(_DEBUG)
printf("\n\nfnTepLD_Strings-----------------------------\n");
#endif
	if (ioStream != NULL)
	{
#if defined(_DEBUG)
printf("ioStream != NULL\n");
#endif
		lpBuf = fnTepLD_Read(ioStream, dwFlags, lpBuf);
		if (lpBuf != NULL && dwFlags & LD_FULL && !fnTepLD_EOF(ioStream))
			((pTepDataString)lpBuf)->NEXT = fnTepLD_Read(ioStream, dwFlags, lpBuf);
#if defined(_DEBUG)
	{
		pTepDataString lpBuff = (pTepDataString)lpBuf;
		if (lpBuff != NULL)
		{
			printf("lpBuff = %08X\n", lpBuff);
			printf("\tID:%08X\n", lpBuff->id);
			printf("\tData:\"%s\"\n", lpBuff->data);
		}
		else
			printf("lpBuff = NULL\n");
	}
#endif
	}

	return lpBuf;
}



/*FUNCTION:     fnTepLD_Clue
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf             	successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for parsing data from character file(s)
*/
void *fnTepLD_Clue(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
	BOOL fRet = FAILURE;

#if defined(_DEBUG)
printf("\n\nfnTepLD_Clue-----------------------------\n");
#endif
	if (ioStream != NULL)
	{
#if defined(_DEBUG)
printf("ioStream != NULL\n");
#endif
		lpBuf = fnTepLD_Read(ioStream, dwFlags, lpBuf);
		if (lpBuf != NULL && dwFlags & LD_FULL && !fnTepLD_EOF(ioStream))
			((pTepClueData)lpBuf)->NEXT = fnTepLD_Read(ioStream, dwFlags, lpBuf);
#if defined(_DEBUG)
	{
		pTepClueData lpBuff = (pTepClueData)lpBuf;
		if (lpBuff != NULL)
		{
			printf("lpBuff = %08X\n", lpBuff);
			printf("\tID:%08X\n", lpBuff->id);
			printf("\tAttached To:%08X\n", lpBuff->attachedto);
			printf("\tDetails:\"%s\"\n", lpBuff->details);
		}
		else
			printf("lpBuff = NULL\n");
	}
#endif
	}

	return lpBuf;
}



/*FUNCTION:     fnTepLD_Command
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf             	successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  function for parsing data from character file(s)
*/
void *fnTepLD_Command(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
	BOOL fRet = FAILURE;

#if defined(_DEBUG)
printf("\n\nfnTepLD_Command-----------------------------\n");
#endif
	if (ioStream != NULL)
	{
#if defined(_DEBUG)
printf("ioStream != NULL\n");
#endif
#if defined(_DEBUG)
printf("calling read...\n");
#endif
		lpBuf = fnTepLD_Read(ioStream, dwFlags, lpBuf);
		if (lpBuf != NULL && dwFlags & LD_FULL && !fnTepLD_EOF(ioStream))
			((pTepDataString)lpBuf)->NEXT = fnTepLD_Read(ioStream, dwFlags, lpBuf);
#if defined(_DEBUG)
	{
		pTepDataString lpBuff = (pTepDataString)lpBuf;
		if (lpBuff != NULL)
		{
			printf("lpBuff = %08X\n", lpBuff);
			printf("\tID:%08X\n", lpBuff->id);
			printf("\tData:\"%s\"\n", lpBuff->data);
		}
		else
			printf("lpBuff = NULL\n");
	}
#endif
	}

	return lpBuf;
}



/*FUNCTION:     fnTepLoadDatabase
 *PARAMETERS:   FILE *ioStream              the file you want to read from
 *				DWORD dwFlags               bit flags for the format of the function
 *              void *lpBuf            		memory space you want the data placed, if this is NULL the function will allocate the appropriate space
 *RETURN VALUE: void *lpBuf             	successful completion
 *              void *NULL             		a part of the function or one of its children FAILURE
 *SIDE EFFECTS: loads data from file into memory
 *DESCRIPTION:  Calling function for parsing and loading the database into memory.
*/
void *fnTepLoadDatabase(FILE *ioStream, DWORD dwFlags, void *lpBuf)
{
    BOOL fRet = FAILURE;
    void *lpRet = lpBuf;

#if defined(_DEBUG)
if (ioStream != NULL)
{
	printf("fnTepLoadDatabase::FILE structure\n");
	printf("    _ptr:  %08X\n", ioStream->_ptr);
	printf("    _cnt:  %08d\n", ioStream->_cnt);
	printf("    _base: %08X\n", ioStream->_base);
	printf("    _flag: %08X\n", ioStream->_flag);
	printf("    _file: %08X\n", ioStream->_file);
	printf("    _charbuf:   %08X\n", ioStream->_charbuf);
	printf("    _bufsiz:    %08X\n", ioStream->_bufsiz);
	printf("    _tmpfname:  %s\n", ioStream->_tmpfname);
}
else
	printf("ioStream == NULL! line %d\n", __LINE__);
#endif

    if (ioStream != NULL)
    {
#if defined(_DEBUG)
printf("ioStream != NULL\n");
#endif
/*		if (dwFlags & LD_GAMESTATE)
		{
#if defined(_DEBUG)
printf("dwFlags has LD_GAMESTATE\n");
#endif
			if (lpRet == NULL)
			{
				lpRet = malloc(sizeof(TepCurState));
				if (lpRet != NULL)
					memset(lpRet, 0, sizeof(lpRet));
			}

			lpRet = fnTepLoadDatabase(ioStream, TEP_CHARACTER | LD_FULL, &((pTepCurState)lpRet)->liveList);

			if (lpRet != NULL)
				lpRet = fnTepLoadDatabase(ioStream, TEP_ROOM | LD_MAPDATA | LD_FULL, &((pTepCurState)lpRet)->FullMap);

			if (lpRet != NULL)
				lpRet = fnTepLoadDatabase(ioStream, TEP_ROOM | LD_DYNDATA | LD_FULL, &((pTepCurState)lpRet)->mapLite);

			if (lpRet != NULL)
				lpRet = fnTepLoadDatabase(ioStream, LD_STRINGS | LD_FULL, &((pTepCurState)lpRet)->keywords);

			if (lpRet != NULL)
				lpRet = fnTepLoadDatabase(ioStream, TEP_TALK | LD_FULL, &((pTepCurState)lpRet)->genConv);

			if (lpRet == NULL)
			{
				if (lpRet != NULL)
				{
					if (((pTepCurState)lpRet)->liveList != NULL)
						free(&((pTepCurState)lpRet)->liveList);
					if (((pTepCurState)lpRet)->FullMap != NULL)
						free(&((pTepCurState)lpRet)->FullMap);
					if (((pTepCurState)lpRet)->mapLite != NULL)
						free(&((pTepCurState)lpRet)->mapLite);
					if (((pTepCurState)lpRet)->keywords != NULL)
						free(&((pTepCurState)lpRet)->keywords);
					if (((pTepCurState)lpRet)->genConv != NULL)
						free(&((pTepCurState)lpRet)->genConv);

					free(lpRet);
					lpRet = NULL;
				}
			}
			else
				fRet = SUCCESS;
		}
		else*/ if (dwFlags & TEP_CHARACTER)
		{
#if defined(_DEBUG)
printf("dwFlags has TEP_CHARACTER\n");
#endif
			lpRet = fnTepLD_Character(ioStream, dwFlags, lpRet);
		}
		else if (dwFlags & TEP_ROOM)
		{
			lpRet = fnTepLD_Location(ioStream, dwFlags, lpRet);
		}
		else if (dwFlags & TEP_TALK)
		{
			lpRet = fnTepLD_Conversation(ioStream, dwFlags, lpRet);
		}
		else if (dwFlags & TEP_DEATH)
		{
			lpRet = fnTepLD_Death(ioStream, dwFlags, lpRet);
		}
		else if (dwFlags & LD_STRINGS)
		{
			lpRet = fnTepLD_Strings(ioStream, dwFlags, lpRet);
		}
		else if (dwFlags & TEP_CLUE)
		{
			lpRet = fnTepLD_Clue(ioStream, dwFlags, lpRet);
		}
		else if (dwFlags & TEP_COMMAND)
		{
			lpRet = fnTepLD_Command(ioStream, dwFlags, lpRet);
		}
	}
	else
	{
		lpRet = NULL;
	}


    return lpRet;
}

#ifdef _DEBUGDEFINED
#undef _DEBUGDEFINED
#define _DEBUG
#endif

/*
int main (int argc, char **argv, char **envp)
{
	FILE *ioStream = NULL;
	void *lpBuffer = NULL, *lpBack = NULL;

	#if defined(_DEBUG)
	printf("attemptimg to load '%s'...\n", argv[1]);
	#endif

	/* this is the calling that you have to do */
//	ioStream = fopen(argv[1], "rb");
	/* do NOT call with LD_FULL in dwFlags (2nd) field.. it hangs... */
//	printf("\n\n\nresult %08X\n\n",lpBuffer = fnTepLoadDatabase(ioStream, TEP_CHARACTER, lpBuffer));
//	fclose(ioStream);



/*	if (lpBuffer != NULL)
		free(lpBuffer);
	return 0;
}*/


/* EOF */
