#include "tep_incs.h"


BOOL fnInterpretTalk(TepCurState *gameState, TepID charTalking)
{
	TepParse parseStruct;						/* Struct that ParseText will use to determine relevant player input */
												/* data. */
	TepDynRoom *pCurDRoom = NULL;				/* pointer to the current rooms dynamic map data */
	TepChar *pCharTalking = NULL;				/* A receptacle to store a pointer to the character */
												/* that is currently speaking to the player */	
	char szPlayerInput[INPUT_SIZE];				/* a string to hold the player's input */
	BOOL fCharAvailable = FALSE;


	/* were we passed a character ID? */
	if( charTalking != PARSE_UNUSED )
	{
		/* get the current dynamic room */
		if( ( pCurDRoom = fnTepDynRoomSearch( gameState->mapLite, gameState->currentRoom->id ) ) != NULL)
		{
			/* is the character in the room? */
			if( fnTepIDSearch( pCurDRoom->people, charTalking ) != NULL )
			{
				/* ok who is the player talking to? */
				if( ( pCharTalking = fnTepCharSearch( gameState->liveList, charTalking ) ) != NULL )
				{
					fCharAvailable = TRUE;
				}
			}
		}
	}

	
	/* so do we have anyone to talk to? */
	if( !fCharAvailable )
	{
		/* we're talking to a dead or invalid character so fail */
		fnPrintOut(NOPAUSE, NULL,NULL,1, " # You cannot talk to them.\n");
		//printf( "you can't talk to them." );
		return FAILURE;
	}

	/* greet the player */
	fnGreet( gameState, pCharTalking->id );

	/* check for verbal clues */
	{
		TepIDUnNode *pCharClueList = pCharTalking->clues;
		TepCurMurder *pMList = gameState->currentMurder;

		/* loop through the character's clue list */
		for( ; pCharClueList != NULL; pCharClueList = pCharClueList->NEXT )
		{
			/* is the current clue a verbal clue? */
			if( pCharClueList->NODE & CLUE_PERPVERBAL )
			{
				TepClueData *pClue;

				if( pMList != NULL )
				{
					while( pMList->NEXT != NULL )
					{
						pMList = pMList->NEXT;
					}
				}

				/* get the actual clue data */
				if( ( pClue = fnClueDataSearch( pMList->death->clues, pCharClueList->NODE, 0 ) ) != NULL )
				{
					/* display the clue */
					fnPrintOut(NOPAUSE, NULL, NULL, 2, " #", pClue->details);

					/* collect the clue */
					fnTepEvidenceAppend( &gameState->player->evidenceGeneric, pClue->id, TEP_CHARACTER | CHAR_GEN, pCharTalking->id );

					/* delete the evidence from the characer so this doesn't happen twice */
					fnTepIDDelete( &pCharTalking->clues, pCharClueList );

					/* tell the user that the clue is in their general evidence */
					fnPrintOut(NOPAUSE, NULL, NULL, 3, " #The statement has been recorded in your GENERAL evidence as ", pClue->keyword, ".");
				}
			}
		}
	}/* end verbal clue handler */

	for(;;)
	{
		/* show the prompt */
		fnIntActPrompt(gameState, pCharTalking->name);

		/* get the player's input and pass it to ParseText */
		gets(szPlayerInput);
		fnParseText(szPlayerInput, &parseStruct, gameState);
		
		parseStruct.Char2 = parseStruct.Char1;
		parseStruct.Char1 = pCharTalking->id;

		#ifdef _DEBUG
			printf("Command: %08x\n", parseStruct.Command );
		#endif

		switch(parseStruct.Command)
		{
			case COM_GREET:
				if( parseStruct.Char1 != PARSE_UNUSED )
				{
					if( fnGreet( gameState, parseStruct.Char1 ) )
					{
						gameState->turnsLeft += ACT_GREET;
					}
				}
				break;

			case COM_ASK:
				{
					#if defined(_DEBUG)
					/*
					{
						char *pszStr = NULL;
						if (fnTepCharSearch(gameState->liveList, parseStruct.Char2))
							pszStr = fnTepCharSearch(gameState->liveList, parseStruct.Char2)->name;
						else if (fnTepRoomSearch(gameState->currentRoom, parseStruct.Char2))
							pszStr = gameState->currentRoom->name;

						printf("\n\nAsking %s about\n%s, %08X, %08X\n", pCharTalking->name, pszStr, parseStruct.Clue, parseStruct.Reserved);
					}
					*/
					#endif
					if (fnAsk( gameState, pCharTalking, &parseStruct ))
					{
						gameState->turnsLeft += ACT_ASK;
					}
				}
				break;


			case COM_SHOW:
				{
					/* call fnAsk, check for error, nate 3/17/01 */
					if(fnShow(gameState, parseStruct) != SUCCESS)
					{
						printf("|\n");
					}
					/* increment the time counter, nate 3/17/01 */
					gameState->turnsLeft += ACT_SHOW;	
				}
				break;

			case COM_COMPLIMENT:
				{
					/* call fnCompliment, check for error, nate 3/17/01 */
					if(fnCompliment(gameState, charTalking) != SUCCESS)
					{
						printf("fnCompliment buggered up in tep_int_talk\n");
					}
					/* increment the time counter, nate 3/17/01 */
					gameState->turnsLeft += ACT_COMPLIMENT;			
				}
				break;

			case COM_INSULT:
				{
					/* call fnInsult, check for error, nate 3/17/01 */
					if(fnInsult(gameState, charTalking) != SUCCESS)
					{
						printf("fnInsult buggered up in tep_int_talk\n");
					}
					/* increment the time counter, nate 3/17/01 */
					gameState->turnsLeft += ACT_INSULT;
				}
				break;

			case COM_GOODBYE:
				{
					goto BYEBYE;
				}
				break;

			default:
				{
					break;
				}
		}/* end switch */

		continue;

		BYEBYE:
			break;
	}/* end for */

	return SUCCESS;
}