#include "tep_incs.h"



BOOL fnNewTurn( TepCurState *pGameState )
{
    TepChar *pMurderer = NULL;              /* pointer to this turn's murderer */
    TepChar *pVictim = NULL;                /* pointer to this turn's victim */

    TepIDUnNode *pID = NULL;                /* used to move through lists of TepIDs */

    TepCurMurder *pCurMurder = NULL;        /* pointer to the current murder struct */

    TepDeath *pDeathList = NULL;
    TepDeath *pCurDeath = NULL;             /* pointer to the current death struct */
    TepDeath *pDeath = NULL;                /* used to step through the list */

    TepClueData *pClueList = NULL;          /* pointer to the list of all game clues */
    TepClueData *pPickedClues = NULL;       /* pointer to today's red herrings */

	BOOL fRet = TRUE;						/* load and keyword function return values */



    /* reset the game turn time */
    pGameState->turnsLeft = BASE_STARTTIME;



    /***********************************************/
    /* load the deaths and clues off of pDeathList and pClueList */
    /***********************************************/

	/* load deaths */
	if( fnTepEditDeath(&pDeathList, IPS_DEATH) == FAILURE )
	{
		printf("Could not load the death list.");
		return FAILURE;
	}

	/* load clues */
	if( fnTepEditClue(&pClueList, IPS_CLUE) == FAILURE )
	{
		printf("Could not load the clue list.");
		return FAILURE;
	}

	/* ALSO MUST GENERATE KEYWORDS FO THE CLUES THAT ARE PICKED
        AFTER GETCLUE IS CALLED IN SET POSITIONS IS A GOOD PLACE FOR IT */



    /* pick a random death */
    pCurDeath = fnPickRandDeath( pGameState, pDeathList );
//	 printf("The current murder is:\n    %s\n", pCurDeath->desc);

    /* pick a random murderer */
    pMurderer = fnPickRandBadChar( pGameState );
    #if defined(_DEBUG)
    printf("%s is the murderer.\n", pMurderer->name);
    #endif

    /* pick a random victim */
	pVictim = fnPickRandGoodChar( pGameState );
	#if defined(_DEBUG)
	printf("%s is the victim.\n", pVictim->name);
	#endif


/* test for printout gender *********************
#if defined(_DEBUG)
	printf("\n\n0x%08x\n", CharGenderFind(pVictim->id) );
	fnPrintOut(NOPAUSE | CharGenderFind(pVictim->id), NULL, pVictim->name, 3, "TESTING GENDER SUB:", pCurDeath->desc, "\n\n");
#endif
*/


	fnTepWhackRemover(pGameState->liveList, pVictim->id);

    /* use pDeath to point a new death node for the murder */
    pDeath = malloc( sizeof( TepDeath ) );
    if( pDeath == NULL )
    {
        return FAILURE;
    }

    /* init pDeath's memory with the chosen death */
    pDeath->id = pCurDeath->id;
	pDeath->location = pCurDeath->location;
    pDeath->desc = pCurDeath->desc;
    pDeath->conv = pCurDeath->conv;
    pDeath->clues = NULL; // these will be picked later within fnSetPositions because we need the death location first...
    pDeath->NEXT = NULL;

    /*	now NULL out the origional pointers to lists in pCurDeath so they aren't free'd later
        they shouldn't be free'd until the end of the game - waaay after the temporary death list is free'd */
    pCurDeath->desc = NULL;
    pCurDeath->conv = NULL;

    // fnPickHerrings();


    /* append the current murder details to the curMurder list */
    if( fnTepCurMurderAppend( &pGameState->currentMurder, pVictim->id, pMurderer->id, pDeath ) == FAILURE )
    {
        free( pDeath );
        return FAILURE;
    }

    /* kill the victim */
    fnTepCharDelete( &pGameState->liveList, pVictim );

    /* place the death, characters, and clues */
    fnSetPositions( pGameState, pClueList );

	/* generate the keywords for the picked clues */
	TepKeywordGenerator( pGameState, NULL, NULL, pDeath->clues, NULL );


    /* free up the death list... and the unused clues */
    fnTepDeathDelete( &pDeathList, NULL );
    fnClueDataDelete( &pClueList, NULL );

    return SUCCESS;
}



TepDeath *fnPickRandDeath( TepCurState *pGameState, TepDeath *pDeathList )
{
    int iRand;                  /* temp integer to store random values */
    int iCount;                 /* temp integer used to store totals of lists */
    int i;

    TepDeath *pCurDeath;        /* pointer to a death */

    /* count the available deaths */
    pCurDeath = pDeathList;
    for( iCount = 0; pCurDeath != NULL; iCount++, pCurDeath = pCurDeath->NEXT );

    /* pick a random death - loop until it's unique */
    do
    {
        /* fill iRand with an appropriate random value */
        iRand = rand() % iCount;

        /* pick the death... */
        pCurDeath = pDeathList;
        for( i = 0; i < iRand; i++, pCurDeath = pCurDeath->NEXT );

    }while( fnTepCurMurderSearch( pGameState->currentMurder, pCurDeath->id ) != NULL );

    /* return the death we've picked */
    return pCurDeath;
}



BOOL fnGetClue( TepClueData **ppMasterList, TepClueData **ppNewList, TepID ClueID, TepID AttachedID )
{
    TepClueData *pMatch;

    /* get the character clue and add it to the list */
    if( (pMatch = fnClueDataSearch( *ppMasterList, ClueID, AttachedID ) ) != NULL )
    {
        /* append it to the list */
        if( fnClueDataAppend( ppNewList, pMatch->id, pMatch->attachedto, pMatch->details, pMatch->keyword ) == FAILURE )
        {
            return FAILURE;
        }

        /* null out the existing string pointers */
        pMatch->details = NULL;
        pMatch->keyword = NULL;

        /* delete the clue from the clue list */
        fnClueDataDelete( ppMasterList, pMatch );
    }

    return SUCCESS;
}

//BOOL fnPickHerrings( TepClueData **ppMasterList, TepClueData **ppNewList, TepID ClueID, TepID AttachedID );


int fnTallyMap( TepRoom *Map )
{
    TepRoom *pCur = Map;
    int tally = 0;

    /* loop through the list and increment tally each time */
    while( pCur != NULL )
    {
        tally++;
        pCur = pCur->NEXT;
    }

    /* return the number of rooms */
    return tally;
}




TepRoom *fnPickRandRoom( TepCurState *pGameState, int iRoomCount )
{
    int i, iRand;
    TepRoom *pPickedRoom = pGameState->FullMap;

    /* pick a random room */
    iRand = rand() % iRoomCount;
    #if defined(_DEBUG)
	/*
    printf("[fnPickRandRoom]::iRoomCount=%d\n", iRoomCount);
	*/
    #endif

    /* get a pointer to the random room */
    for( i = 0; i < iRand; i++, pPickedRoom->NEXT );

    #if defined(_DEBUG)
	/*
    printf("[fnPickRandRoom]::pPickedRoom=%08X\n", pPickedRoom);
    printf("[fnPickRandRoom]::pPickedRoom->name=%s\n", pPickedRoom->name);
    printf("[fnPickRandRoom]::pPickedRoom->desc=%s\n", pPickedRoom->desc);
    printf("[fnPickRandRoom]::pPickedRoom->id=%08X\n", pPickedRoom->id);
	*/
    #endif

    return pPickedRoom;
}




TepIDUnNode *fnPickRandID( TepIDUnNode *pIDList )
{
    int iNum, iRand;
    TepIDUnNode *pCur = pIDList;

    /* count the total elements in the list */
    for( iNum = 0; pCur != NULL; iNum++, pCur = pCur->NEXT );


    if( iNum > 0 )
    {
        /* pick a random item in the list */
        iRand = rand() % iNum;
        for( pCur = pIDList, iNum = 0; iNum < iRand; iNum++, pCur = pCur->NEXT );
    }

    return pCur;
}




BOOL fnSetPositions( TepCurState *pGameState, TepClueData *pClueList )
{
    int iMapSize;

    TepRoom *pRoom = NULL;

    TepDynRoom *pDeathRoom = NULL;
    TepDynRoom *pDynRoom = NULL;

    TepIDUnNode *pIDList = NULL;
    TepIDUnNode *pID = NULL;

    TepChar *pCharacter = NULL;

    TepClueData *pClue = NULL;
    TepClueData *pPickedClues = NULL;

    TepCurMurder *pCurMurder = NULL;

    /* get the current murder */
    pCurMurder = pGameState->currentMurder;
    if( pCurMurder != NULL )
    {
        while( pCurMurder->NEXT != NULL )
        {
            pCurMurder = pCurMurder->NEXT;
        }
    }
    else
    {
        /* there is no murder! */
        return FAILURE;
    }

	/* if the murder is not location specific... */
	if( pCurMurder->death->location == 0 )
	{
		/* pick a random location */
		/* get the size of the current map */
		iMapSize = fnTallyMap( pGameState->FullMap );

		do
		{
			/* pick a room */
			pRoom = fnPickRandRoom( pGameState, iMapSize );
		}while( !(pRoom->id | ROOM_OBJ) ); /* loop until the room is not an object */

		/* set the murder location in the death struct */
		pCurMurder->death->location = pRoom->id;
	}
	/* if the murder is location specific - get the proper room */
	else
	{
		pRoom = fnTepRoomSearch( pGameState->FullMap, pCurMurder->death->location );
	}


    /* get the clue for a loc that the murdereer frequents */
    pCharacter = fnTepCharSearch( pGameState->liveList, pCurMurder->murderer );
    pID = fnPickRandID( pCharacter->locations );
    fnGetClue( &pClueList, &pCurMurder->death->clues, TEP_CLUE | CLUE_LOCATION, pID->NODE );

	/* append this to the murder loc */
	fnTepIDAppend( &pIDList, pCurMurder->death->clues->id );


    /* get all of the clues now that there's a death location */
    fnGetClue( &pClueList, &pCurMurder->death->clues, TEP_CLUE | CLUE_PERPITEM, pCurMurder->murderer );
    fnGetClue( &pClueList, &pCurMurder->death->clues, TEP_CLUE | CLUE_PERPVERBAL, pCurMurder->murderer );
    fnGetClue( &pClueList, &pCurMurder->death->clues, TEP_CLUE | CLUE_LOCATION, pRoom->id );
    fnGetClue( &pClueList, &pCurMurder->death->clues, TEP_CLUE | CLUE_FROMDEATHONPERP, pCurMurder->death->id );
    fnGetClue( &pClueList, &pCurMurder->death->clues, TEP_CLUE | CLUE_FROMDEATHATLOCA, pCurMurder->death->id );


    /* start an object list for the dynroom */
    fnTepIDAppend( &pIDList, pCurMurder->death->id );

    /* place clues at the murder location */
    /* place the murderer's item clue */
    pClue = fnClueDataSearch( pCurMurder->death->clues, TEP_CLUE | CLUE_PERPITEM, pCurMurder->murderer );

    /* append it to the witness */
    if( pClue != NULL )
    {
        fnTepIDAppend( &pIDList, pClue->id );
    }

    /* add a dynamic room for the room with the death */
    fnTepDynRoomAppend( &pGameState->mapLite, pRoom->id, pRoom, NULL, pIDList );


    /* place characters around the island */
    /* start at the top of the list */
    pCharacter = pGameState->liveList;

    /* loop until the end of list */
    while( pCharacter != NULL )
    {
        #if defined(_DEBUG)
		printf("\t%s\n", pCharacter->name);
		#endif
        /* pick a possible location for the character - does not care about murder location!!! */
        pID = fnPickRandID( pCharacter->locations );

        /* check if that location already has a dyn map */
        pDynRoom = fnTepDynRoomSearch( pGameState->mapLite, pID->NODE );


        if( pDynRoom == NULL )
        {
            /* create a dyn map node */
            /* find the full room data */
            pRoom = fnTepRoomSearch( pGameState->FullMap, pID->NODE );

            /* start an object list for the dynroom */
            pIDList = NULL;
            fnTepIDAppend( &pIDList, pCharacter->id );

            /* create the dyn room and add the people list */
            fnTepDynRoomAppend( &pGameState->mapLite, pRoom->id, pRoom, pIDList, NULL );
        }
        else
        {
            /* append the character to the existing node */
            fnTepIDAppend( &pDynRoom->people, pCharacter->id );
        }

		/* set the character's compliment counter - if the character really doesn't like the player subtract one */
		pCharacter->complimentCounter = (rand() % BASE_COMP_HIGH) + BASE_COMP_ADJUST;
		if( pCharacter->like < BASE_COMP_HATELEVEL && pCharacter->complimentCounter > 0 )
		{
			pCharacter->complimentCounter -= BASE_COMP_ADJUST;
		}

		/* increment the character */
		pCharacter = pCharacter->NEXT;
    }



    /* place all the character clues */
    /* place a verbal clue on a witness */
    pCharacter = fnPickRandGoodChar( pGameState );

    /* find the verbal clue */
    pClue = fnClueDataSearch( pCurMurder->death->clues, TEP_CLUE | CLUE_PERPVERBAL, pCurMurder->murderer );

    /* append it to the witness */
    if( pClue != NULL )
    {
        fnTepIDAppend( &pCharacter->clues, pClue->id );
    }


    /* place the clues on the murderer - find the murderer first */
    pCharacter = fnTepCharSearch( pGameState->liveList, pCurMurder->murderer );


    /* Stick the death specific clue on the perp */
    /* find the clue first */
    pClue = fnClueDataSearch( pCurMurder->death->clues, TEP_CLUE | CLUE_FROMDEATHONPERP, pCurMurder->death->id );

    /* append it to the murderer */
    if( pClue != NULL )
    {
        fnTepIDAppend( &pCharacter->clues, pClue->id );
    }

    /* pCharacter points to the murderer so we can place the rest of their clues now */
    /* get a random location the murderer frequents */
    pID = fnPickRandID( pCharacter->locations );

    /* check if that location already has a dyn map */
    pDynRoom = fnTepDynRoomSearch( pGameState->mapLite, pID->NODE );

    /* pick the clue */
    pClue = fnClueDataSearch( pCurMurder->death->clues, TEP_CLUE | CLUE_LOCATION, pCurMurder->death->location );

	if( pClue != NULL )
	{
		if( pDynRoom == NULL )
		{
			/* create a dyn map node */
			/* find the full room data */
			pRoom = fnTepRoomSearch( pGameState->FullMap, pID->NODE );

			/* start an object list for the dynroom */
			pIDList = NULL;
			fnTepIDAppend( &pIDList, pClue->id );

			/* create the dyn room and add the people list */
			fnTepDynRoomAppend( &pGameState->mapLite, pRoom->id, pRoom, NULL, pIDList );
		}
		else
		{
			/* append the character to the existing node */
			fnTepIDAppend( &pDynRoom->objs, pClue->id );
		}
	}



    /* get a random location the murderer frequents */
    pID = fnPickRandID( pCharacter->locations );

    /* check if that location already has a dyn map */
    pDynRoom = fnTepDynRoomSearch( pGameState->mapLite, pID->NODE );

    /* pick the clue */
    pClue = fnClueDataSearch( pCurMurder->death->clues, TEP_CLUE | CLUE_FROMDEATHATLOCA, pCurMurder->death->id );

	if( pClue != NULL )
	{
		if( pDynRoom == NULL )
		{
			/* create a dyn map node */
			/* find the full room data */
			pRoom = fnTepRoomSearch( pGameState->FullMap, pID->NODE );

			/* start an object list for the dynroom */
			pIDList = NULL;
			fnTepIDAppend( &pIDList, pClue->id );

			/* create the dyn room and add the people list */
			fnTepDynRoomAppend( &pGameState->mapLite, pRoom->id, pRoom, NULL, pIDList );
		}
		else
		{
			/* append the character to the existing node */
			fnTepIDAppend( &pDynRoom->objs, pClue->id );
		}
	}

    return TRUE;
}
