#include "tep_incs.h"


/* Programmer Name: Nathaniel Cleveland
 * Date Created: January 26th, 2000 
 * Program Updated by:			Nate, April 4th, 2001
 *								Nate Cleveland, February 8th, 2000
 *								Nate Cleveland, February 3rd, 2000
 *
 * Program Description:			This program takes a line of input, and a pointer to a TepParse struct and fills that 
								struct with all of the reasonable commands from the line of input.
 *
 * Program dependacies:			"tep_typs.h", "ParseText.h", fnInputListSearch, 
 *								fnParseText_PrepareString, fnAppendTepDataString
 * Takes:						pChar cpInputString, the line of input from the player
 * Gives:						A boolean value, 
 *								TRUE means that we did find a match in the inputString
 *								FALSE means we did not.
 *				
 */

/************************ DEBUG STUFF ****************************/
/*
TepDataString ListRoot;
TepDataString Character;
TepDataString Clue;
TepDataString Room;
TepDataString Command;
TepDataString Character2;
TepDataString Character3;
TepDataString Command2;
TepDataString Command3;
TepDataString Command4;
TepDataString Command5;


void SetGlobals(void)
{
	ListRoot.id = 0;
	ListRoot.data = "This is the list root";
	ListRoot.NEXT = &Character;

	Character.id = TEP_CHARACTER +1;
	Character.data = "TOTOK";
	Character.NEXT = &Clue;

	Clue.id = TEP_CLUE + 1;
	Clue.data = "KNIFE";
	Clue.NEXT = &Room;

	Room.id = TEP_ROOM + 1;
	Room.data = "HUT1";
	Room.NEXT = &Command;

	Command.id = COM_NORTH;
	Command.data = "NORTH";
	Command.NEXT = &Character2;

	Character2.id = TEP_CHARACTER + 2;
	Character2.data = "RUSHI";
	Character2.NEXT = &Character3;

	Character3.id = TEP_CHARACTER + 3;
	Character3.data = "COMAN";
	Character3.NEXT = &Command2;

	Command2.id = COM_GO;
	Command2.data = "GO";
	Command2.NEXT = &Command3;

	Command3.id = COM_SOUTH;
	Command3.data = "SOUTH";
	Command3.NEXT = &Command4;

	Command4.id = COM_EAST;
	Command4.data = "EAST";
	Command4.NEXT = &Command5;

	Command5.id = COM_WEST;
	Command5.data = "WEST";
	Command5.NEXT = NULL;
}
*/
/******************************************************/

BOOL fnParseText(pChar cpInputString, TepParse *TepPar, TepCurState *pGameState)
{
	ULONG uHitCounter = 0;						/* Number of hits fnInputListSearch returns */
	TepDataString *pmsgList = NULL;				/* list of messages from fnInputListSearch */
	BOOL bErrorCheck;							/* bool to check error with */
	TepDataString *pCommand = NULL;
	TepDataString *pmsgListRoot = NULL;

	//SetGlobals();

	TepPar->Command = PARSE_UNUSED;
	TepPar->Char1 = PARSE_UNUSED;
	TepPar->Char2 = PARSE_UNUSED;
	TepPar->Clue = PARSE_UNUSED;
	TepPar->Reserved = PARSE_UNUSED;

		/* prepare the string */
	fnParseText_PrepareString(cpInputString);

		/* find any hits in the string, and get a list of them */
	uHitCounter = fnInputListSearch(pGameState->keywords, &pmsgListRoot, cpInputString);

		/* check to see if we found anything, if we didn't return FALSE */
	if(uHitCounter == 0)
	{
		bErrorCheck = fnTepDataStringDelete(&pmsgListRoot, NULL);	/* clean up */
		return FALSE;											/* exit function */
	}  /* end if */

	//printf("\nmatch commands\n");
	for( pmsgList = pmsgListRoot ;pmsgList != NULL ;pmsgList = pmsgList->NEXT)
	{
		//printf("%s - 0x%x\n", pmsgList->data, pmsgList->id);

			// Break the loop after the first command has been entered, added by Nate 4/4/01 
			if(TepPar->Command != PARSE_UNUSED)
				break;

		if((pmsgList->id & TEP_COMMAND) == TEP_COMMAND)/* A command */
		{


			if(pmsgList->id == (TEP_COMMAND |COM_NORTH))
			{
				TepPar->Command = COM_GO;
				TepPar->Reserved = TEP_COMMAND | COM_NORTH;
				continue;
			} /* end north if */
			else if(pmsgList->id == (TEP_COMMAND |COM_SOUTH))
			{
				TepPar->Command = COM_GO;
				TepPar->Reserved = TEP_COMMAND | COM_SOUTH;
				continue;
			}/* end south else if */
			else if(pmsgList->id == (TEP_COMMAND | COM_WEST))
			{
				TepPar->Command = COM_GO;
				TepPar->Reserved = TEP_COMMAND | COM_WEST;
				continue;
			}/* end west else if */
			else if(pmsgList->id == (TEP_COMMAND | COM_EAST))
			{
				TepPar->Command = COM_GO;
				TepPar->Reserved = TEP_COMMAND | COM_EAST;
				continue;
			}/* end east else if */

			TepPar->Command = pmsgList->id;		
		}  /* end Command else if */

	}

	//printf("\nmatch the rest\n");
	for( pmsgList = pmsgListRoot ;pmsgList != NULL ;pmsgList = pmsgList->NEXT)
	{
		//printf("%s - 0x%x\n", pmsgList->data, pmsgList->id);

		if((pmsgList->id & TEP_CHARACTER) == TEP_CHARACTER)	/* if true then this is a character */
		{
			if(TepPar->Char1 == PARSE_UNUSED)		/* If TepPar is 0 (or null) then char1 is not already in use */
			{
				TepPar->Char1 = pmsgList->id;
			}  /* end if */				
			else						/* If wasn't 0 then it was already filled and we should now fill char2 */
			{
				TepPar->Char2 = pmsgList->id;
			} /* end else */
		}  /* end Char if */

		else if((pmsgList->id & TEP_CLUE) == TEP_CLUE)		/* A clue */
		{
			TepPar->Clue = pmsgList->id;
		}  /* end Clue else if */

		else if((pmsgList->id & TEP_ROOM) == TEP_ROOM)		/* A location, which will be stored in Reserved */
		{
			pCommand = fnTepDataStringSearch( pmsgListRoot, TepPar->Command );
			if( pCommand != NULL )
			{
				//printf("\npCommand - 0x%x\n", pCommand->id);
				if( (pCommand->id == COM_GO) && (pmsgList->id & ROOM_ROOM) )
				{
					//printf("Tries to match a room 0x%x\n", pmsgList->id);
					TepPar->Reserved = pmsgList->id;
				}
				/* added to make tep_ask have something to be asked about, nate 3/26/01 */
				else if( (pCommand->id == COM_ASK) && (pmsgList->id & ROOM_ROOM) )
				{
				//	printf("pmsgList->id is, 0x%X", pmsgList->id);
					TepPar->Reserved = pmsgList->id;
				}
				else if( (pCommand->id == COM_LOOK) && (pmsgList->id & ROOM_OBJ) )
				{
					//printf("Tries to match an object room 0x%x\n", pmsgList->id);
					if(fnTepIDSearch(pGameState->currentRoom->objs, pmsgList->id) != NULL)
					{
						TepPar->Reserved = pmsgList->id;
					}
				}
			}


		}  /* end locations (aka Reserved) if */
	}
		/* return the bool and clean up */
	/* prevent the deletion of the keword list data strings */
	{
		TepDataString *pThis = pmsgListRoot;

		while( pThis != NULL )
		{
			pThis->data = NULL;
			pThis = pThis->NEXT;
		}
	}

	bErrorCheck = fnTepDataStringDelete(&pmsgListRoot, NULL);
	return TRUE;

}  /* end function */



/* Function Name: fnParseText_PrepareString
 * Date Created: January 26th, 2000 
 * Function Updated by:			Nate Cleveland, February 3rd, 2000
 *
 * Function Description:		This function takes a string of input from ParseText and prepares it.
 *								1st, it uppercaseifies all of the letters. 2nd, it strips out all of the contractions 
 *								and possesives in the string, thus avoiding confusion
 *
 * Function dependacies:		"tep_incs.h"
 * Takes:						pChar cpInputString, a sting of characters entered originally by the user
 * Gives:						Nothing			
 */
void fnParseText_PrepareString(pChar cpInputString)
{
	pChar cpContString = 0;						/* Place to hold the contraction tokens */
	uChar cStringofConts[] = P_CONTRACTIONS;		/* Place to hold the string of contractions before I tok them */
	pChar cpStringToCut = 0;					/* Pointer to the string we need to kill */
	int nCounter = 0;							/* Counter for the loop */

			/* Set make all of the characters upper case via toupper, so we can avoid case problems */
	for(nCounter = 0; cpInputString[nCounter] != 0; nCounter++)
	{
		cpInputString[nCounter] = toupper(cpInputString[nCounter]);
	}  /* End for loop */

		/* Take a token and use strstr to see if it is in the file, if it is fill it with memset */
		/* for exp1: tokenize the list of contractions, keep a pointer to the contractions in cpContString */
		/* for exp2: keep looping until the pointer to the string of contractions to be removed is NULL */
		/* for exp3: pass a null to strtok which has the effect of passing a pointer the the next token in the string */
	for(cpContString = strtok(cStringofConts, " " ); cpContString != NULL ; cpContString = strtok(NULL, " "))
	{
			/* Create a loop to go thru and check for the contraction until there aren't any more */
			/* for exp1: call strstr to find a pointer the the first occurance of the contraction */
			/* for exp2: check to see if cpString == 0, if it does break the loop */
			/* for exp3: call strstr find the next occurance of the contraction */
		for(cpStringToCut = strstr(cpInputString, cpContString) ; 
			cpStringToCut != 0 ; 
			cpStringToCut = strstr(cpInputString, cpContString))
		{
				memset(cpStringToCut, '*', strlen(cpContString));
		}/* end inner for */
	}  /* End outer for */
}  /* End function */


/* Function Name: fnInputListSearch
 * Function Creator: John Corpening
 * Date Created: February 8th, 2000 
 * Function Updated by:			Nate Cleveland, February 8th, 2000
 *
 * Function Description:		This function zips thru a list of allowed commands, and sees if any thing the player
 *								typed in is one of them. It then creates a linked list of them of "hits".
 *
 * Function dependacies:		"tep_incs.h"
 * Takes:						TepDataString *pListRoot,	The master linked list of commands 
 *								TepDataString **msgList,	The list it will create
 *								pChar inputString,			Input from the player
 * Gives:						The number of hits it had			
 */
int fnInputListSearch(TepDataString *pListRoot, TepDataString **msgList, pChar inputString)
{
    TepDataString *pCurrNode;
	int iHitCounter = 0;
	BOOL bErrorCheck;

	for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
	{
		if(strstr(inputString,pCurrNode->data) != NULL)
		{
//			printf("MATCH -> ");
			bErrorCheck = fnTepDataStringAppend(msgList, pCurrNode->id, pCurrNode->data);
			iHitCounter++;
		}
//		printf("%s!\n", pCurrNode->data);
	} 	
    return iHitCounter;
}




/**********************
****FOR DEBUG ONLY*****
**********************/
/*
int main (void)
{
*/
//	BOOL CheckBool= TRUE;				/* bool used to check return vals */
//	uChar inputArray[20000];			/* The Input Array used */
//	TepParse *TepPar;					/* the struct that will be filled */

/*	TepPar = calloc(1, sizeof(TepParse));
	while(1)
	{		
*/
		/* Collect the input and test for EOF. */
/*		if (gets(inputArray) == NULL)
			return 0;

		if (inputArray[0] == 0)
			return 0;


		CheckBool = fnParseText(inputArray, TepPar);

		if(CheckBool == FALSE)
		{
			printf("nothing\n");
			return 0;
		}

		printf("\tTotok is %u, Rushi is %u, Coman is %u", TEP_CHARACTER + 1, TEP_CHARACTER + 2, TEP_CHARACTER + 3);
		printf("\n");
		printf("(Command) is %u\n",TepPar->Command);
		printf("(Clue) is %u\n",TepPar->Clue);
		printf("(Char1) is %u\n",TepPar->Char1);
		printf("(Char2) is %u\n", 0,TepPar->Char2);
		printf("(Reserved) is %u\n",TepPar->Reserved);


	}
	free(TepPar);
	return 0;
}
*/


