/**************************************************************
*** VERSION NUMBER 0011
*
*** ORIGINAL AUTHOR INFORMATION ****
*
* Original Author(s):
*   Gino Constantini    <mastergcc@aol.com>
*   Jon Olson           <nfin@monkey18.intranets.com>
*   Matt Phillips       <matt@monkey18.intranets.com>
*
* Location:
*   Digipen Institute of Technology, Redmond Campus.
*
*** CREATION INFORMATION ****
*
* Create Date:
*   November 6, 2000
*
* Revision History: (older entries toward the bottom)
*	2001-03-06 :: jolson :: Addition:
*							- added the location id into the TepDeath structure.
*   2001-02-10 :: mphillip :: Changes:
*                             - fixed an incorrect TepParse struct tag
*                             - removed NEXT pointer from TepParse struct. it's not a list
*   2001-02-05 :: jolson :: Changes:
*                           - changed TepCmds type to a union. It's now bit fields.
*   2001-02-05 :: jolson :: Fixes:
*                           - fixed some bugs created when I removed a bunch of data.
*   2001-02-05 :: jolson :: Changes:
*                           - redefined the FAILED, failed
*   2001-02-03 :: jolson :: Changes:
*                           - Removed all type usage of bidirectional lists
*                           - Converted all types into unidirectional lists
*                           - Removed all older unidirectional lists
*   2001-02-03 :: jolson :: Addition:
*                           - Death Struct, okay for some reason and death struct that was in here vanished. I have replaced it.
*                           - Murder Struct, added unidirectional and bidirectional structures to the mix
*                           Changes:
*                           - GameState, changed variable naming and type for the past death structs from pastDeaths to pastMurders and pTepDeathUnNode to pTepCurMurder respectively.
*   2001-02-03 :: jolson :: Changes:
*                           - TepClueData structure. attachedTo has changed types: from TepID to pTepIDUnNode
*                               this was done because the clue may be attached to several things, including persons, locations, and objects
*                           Additions:
*                           - TepChar structure. added variable 'clues' of type pTepClueData.
*                               this was added to create an easy abstraction to getting clues which are applicable to the current character
*                           Movement:
*                           - All clue structs. moved their position to above the character structs because of a compilation error that would result otherwise.
*   2001-01-29 :: jolson :: name change:
*                           Rametep was spelt wrong in game state. This was fixed by changing the variable name to killersList.
*   2001-01-29 :: jolson :: Minor revision:
*                           the constants file, that was included changed names a couple days ago, I neglected to update this file accordingly.
*   2001-01-24 :: jolson :: Additions:
*                           - TepParse structure
*                           - TepWhack structure
*                           Changes:
*                           - WhackList in TepChar is now of datatype TepWhack, or more specifically, pTepWhack
*   2001-01-16 :: jolson :: Made some corrections and added:
*                           - compliment counter to TepChar struct
*                           - list of evidence shown to a character; destroyed at beginning of each turn/day
*                           renamed file to tep_types.h from mk18_tep.h
*   2000-11-19 :: jolson :: Fixed some bugs that were in the code, added some missing datatypes
*   2000-11-18 :: jolson :: Fixed some bugs that were in the code
*   2000-11-16 :: jolson :: Changed 'Leaf' to 'Node', added more functionality
*   2000-11-15 :: jolson :: Added more known functionality
*   2000-11-11 :: jolson :: Added more of the decided functionality
*   2000-11-08 :: jolson :: Added more structures and typedefs
*   2000-11-07 :: jolson :: Added a bunch of the main structures of the file
*
*** PROGRAM INFORMATION ****
*
* Name :            tep_types.h
*
* Implementation:   <none> (datatypes only)
*
* Objective:
*   The header file for the Tep game.
*
* Methods:
*   declarations and stuffs
*
* Neccessary archives:
*   "constants.h"           custom header file setting all the times that things will take.
*
* Environments:
*   command line
*
* Requirements of the Hardware:
*   none as yet
*
* Restrictions:
*   none as yet
*
* Modes of Use:
*   included in all code to be used with tep
*
* Compiler:
*   Microsoft Visual C++ 6, Service Pack 2
*
* Bibliography:
*   Nate Cleveland      Brain (Wetware)
*   Gino Constantini    Brain (Wetware)
*   Jon Olson           Brain (Wetware)
*   Matt Phillips       Brain (Wetware)
*   Andr LMothe       Text Parsing: How To Guide (intranet)
*   Michael Abrash      Zen Programming (Book)
*
**************************************************************/


#pragma once
/* do this only once */


#if !defined(__TEP_HEADER__) || (__TEP_HEADER__ < 0x0400040B)
#define __TEP_HEADER__ 0x0400040B /* 1.1.11 */
/* Tree 1, File 1, File Version 11 */
/*
Here is how the the bit pattern is broken down:
0xTPFFFMVV
    T + Ph  (6 bits, 64)            :: tree number
    Pl + FFF + Mh (16 bits, 65536)  :: file ID number
    Ml + VV (10 bits, 1024)         :: version of the file

ex ( 0x12345678 equates to file 4.36117.632 or Tree 4, File 36117, v632 )
*/



/****
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$%o,
$ BASIC TYPEDEFS CONDITIONALLY DONE $$$$$$$$$$$$$$$
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$?'
****/

#if !defined(__UCHAR_DEFINED__)
#define __UCHAR_DEFINED__
typedef unsigned char uChar;
#endif

#if !defined(__PCHAR_DEFINED__)
#define __PCHAR_DEFINED__
typedef unsigned char *pChar;
#endif

#if !defined(__PBYTE_DEFINED__)
#define __PBYTE_DEFINED__
typedef unsigned char *pByte;
#endif

#ifndef WINVER
#if !defined(__UINT_DEFINED__)
#define __UINT_DEFINED__
typedef unsigned short UINT;
#endif
#endif

#if !defined(__USHORT_DEFINED__)
#define __USHORT_DEFINED__
typedef unsigned short USHORT;
#endif

#if !defined(__UWORD_DEFINED__)
#define __UWORD_DEFINED__
typedef unsigned short UWORD;
#endif

#if !defined(__ULONG_DEFINED__)
#define __ULONG_DEFINED__
typedef unsigned long ULONG;
#endif

#if !defined(__DWORD_DEFINED__)
#define __DWORD_DEFINED__
typedef unsigned long DWORD;
#endif

#ifndef WINVER
#if !defined(__BOOL_DEFINED__)
#define __BOOL_DEFINED__
typedef char BOOL;
#endif
#endif

#ifndef WINVER
#if !defined(__bool_DEFINED__)
#define __bool_DEFINED__
typedef char bool;
#endif
#endif

#if !defined(__TRUE_DEFINED__)
#define __TRUE_DEFINED__
#define TRUE 1
#endif

#if !defined(__FALSE_DEFINED__)
#define __FALSE_DEFINED__
#define FALSE 0
#endif

#if !defined(__FAILURE_DEFINED__)
#define __FAILURE_DEFINED__
#define FAILURE -1
#endif

#if !defined(__SUCCESS_DEFINED__)
#define __SUCCESS_DEFINED__
#define SUCCESS 1
#endif

#if !defined(__true_DEFINED__)
#define __true_DEFINED__
#define true 1
#endif

#if !defined(__false_DEFINED__)
#define __false_DEFINED__
#define false 0
#endif

#if !defined(__failure_DEFINED__)
#define __failure_DEFINED__
#define failure -1
#endif

#if !defined(__success_DEFINED__)
#define __success_DEFINED__
#define success 1
#endif

#if !defined(__PVOID_DEFINED__)
#define __PVOID_DEFINED__
typedef void *PVOID;
#endif


/****
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$%o,
$ TEXTUAL SUBST $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$?'
****/

/* OBJECT TYPE MASTER ID NUMBERS ******************
    Bit pattern:

        maintype        31 - 26 bits (6)
        secondary       25 - 16 bits (10)
        unique          15 - 0  bits (16)
        ---------------------------------
TOTAL                                 32 bits
**************************************************/
#define TEP_ROOM                0x04000000
#define TEP_CHARACTER           0x08000000
#define TEP_DEATH               0x10000000
#define TEP_CLUE                0x20000000
#define TEP_COMMAND             0x40000000
#define TEP_TALK                0x80000000

/* OBJECT TYPE SECONDARY CATEGORIES **************/
#define ROOM_ROOM				0x00010000	/*Added by Nate and John 2/13/01*/
#define ROOM_OBJ				0x00020000	/*Added by Nate and John 2/13/01*/

#define CLUE_PERPITEM           0x00010000
#define CLUE_PERPLOCA           0x00020000
#define CLUE_PERPVERBALSIGHT    0x00030000
#define CLUE_PERPVERBALSOUND    0x00040000
#define CLUE_FROMDEATHONPERP    0x00050000
#define CLUE_FROMDEATHATLOCA    0x00060000
#define CLUE_TYPEDEATHONPERP    0x00070000
#define CLUE_TYPEDEATHATLOCA    0x00080000
#define CLUE_HERRING			0x00100000  /* added by Matt 3/3/01 */

#define TALK_GENINSULT          0x00010000
#define TALK_GENCOMPLIAMENT     0x00020000
#define TALK_HELLO              0x00030000
#define TALK_GOODBYE            0x00040000
#define TALK_ANSQUERY           0x00050000
#define TALK_ANSEVIDENCEYES     0x00060000
#define TALK_ANSEVIDENCENO      0x00070000
#define TALK_CLUESLOCATION      0x00080000
#define TALK_CLUESCHARACTER     0x00090000
#define TALK_INIT               0x000A0000
#define TALK_GENFAILURE         0x000B0000
#define TALK_ANSEVIDENCEHELLYES	0x000C0000	/*Added by John the elder 2/16/01*/
#define TALK_ANSEVIDENCEHELLNO	0x000D0000	/*Added by John the elder 2/16/01*/
#define TALK_EODINTRO			0x000E0000	/*Added by John the elder 2/16/01*/
#define TALK_EODHILLRUN			0x00A10000	/*Added by John the elder 2/16/01*/
#define TALK_EODHAPPYSPEECH		0x00A20000	/*Added by John the elder 2/16/01*/
#define TALK_EODSADSPEECH		0x00A30000	/*Added by John the elder 2/16/01*/
#define TALK_BOGINTROSPEECH		0x00A40000	/*Added by Nate 2/16/01*/
#define TALK_EODACCUSATION		0x00A50000  /*Added by John the elder 2/17/01*/
#define TALK_EODCRUMBLE			0x00A60000  /*Added by John the elder 2/17/01*/
#define TALK_EODNOCRUMBLE		0x00A70000	/*Added by John the elder 2/17/01*/
#define TALK_EODCLOSINGSPEECH	0x00A80000  /*Added by John the elder 2/17/01*/
#define TALK_SACRIFICE			0x00A90000  /*Added by John the elder 2/17/01*/
#define TALK_EOGWINS			0x00AA0000	/* added by matt 2/26/01 */
#define TALK_EOGLOSE			0x00AB0000  /* added by matt 2/26/01 */
#define TALK_CLUERECALL			0x00AC0000	/* added by Nathan Frost 01/03/03 */
#define TALK_INSULTREACTION		0x00AD0000	/* added by Nathan Frost 01/03/03 */
#define TALK_POSITIVECOMPLIMENT	0x00AE0000	/* added by Nathan Frost 01/03/03 */
#define TALK_NEGATIVECOMPLIMENT	0x00AF0000	/* added by Nathan Frost 01/03/03 */
#define TALK_BOGSHEROESHASPEAKS	0x00B00000	/* added by Nathan Frost 01/03/03 */
#define TALK_TALKABOUTDEAD		0x00B10000	/* added by Nathan Frost 01/03/03 */
#define TALK_BOGINTRODUCTION	0x00B20000	/* added by Nathan Frost 01/03/03 */
#define TALK_BODWAKEUP			0x00B30000	/* added by Nathan Frost 01/03/03 */
#define TALK_EOGWININTRO		0x00B40000	/* added by Nathan Frost 01/03/03 */
#define TALK_EOGWINOUTRO		0x00B50000	/* added by Nathan Frost 01/03/03 */

/* All Purpose Empty Sign */
#define PARSE_UNUSED			0x0000DEAD /*Added by Nate and John 2/13/01*/

/****
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$%o,
$ STRUCTS & OBJECT HANDLES $$$$$$$$$$$$$$$$$$$$$$$$
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$?'
****/

/* ID NUMBER **************************************
    All values stored in this format are unique.
**************************************************/
typedef ULONG TepID;


/* POINTER TO ID NUMBER **************************/
typedef TepID *pTepID;


/* UNIDIRECTIONAL ID NUMBER NODE ******************
    A structure for storage of ID numbers
**************************************************/
typedef struct _TepIDUnNode
{
    TepID                   NODE;       /* current id */
    struct _TepIDUnNode     *NEXT;      /* pointer to next id */
} TepIDUnNode; /* struct _TepIDUnNode */


/* POINTER TO UNIDIRECTIONAL ID NODE *************/
typedef TepIDUnNode *pTepIDUnNode;


/* COMMANDS ALLOWED *******************************
    Bit level list of commands allowed
**************************************************/
typedef union _TepCmds
{
    struct
    {
        ULONG North:1;          /* go north */
        ULONG East:1;           /* go east */
        ULONG South:1;          /* go south */
        ULONG West:1;           /* go west */
        ULONG Look:1;           /* look/examine */
        ULONG Display:1;        /* redisplay a room description - used to be take */
        ULONG Leave:1;          /* leave hut */
        ULONG Put:1;            /* put <object> in evidence [<character>] */
        ULONG Go:1;             /* go somewhere */
        ULONG Help:1;           /* get help */
        ULONG Save:1;           /* save the game */
        ULONG Load:1;           /* load a game */
        ULONG Quit:1;           /* quit the game */
        ULONG Greet:1;          /* greet a character */
        ULONG Insult:1;         /* insult a character */
        ULONG Talk:1;           /* talk to a character */
        ULONG cmdReserved:16;   /* currently unused bits */
    };
    ULONG allCmd;               /* the entire union in one piece */
} TepCmds; /* union _TepCmds */


/* POINTER TO COMMANDS ALLOWED STRUCT *************
    Pointer to bit level list of commands
    Note: this datatype is practically useless and will probably be depricated in a future release.
**************************************************/
typedef TepCmds *pTepCmds;


/***  DATA STRING STRUCTS  ***************************************************
*****************************************************************************/
/* DATA STRUCT ************************************
    A struct for storage of data. This structure
    is to be used primarily in file database data
    strings.
**************************************************/
typedef struct _TepDataString
{
    TepID                   id;         /* ID of the data string */
    pChar                   data;       /* storage data */
    struct _TepDataString   *NEXT;      /* pointer to next DataString */
} TepDataString;


/* POINTER TO DATA STRUCT ************************/
typedef TepDataString *pTepDataString;


/***  PARSE STRUCTS  *********************************************************
*****************************************************************************/
/* PARSING ITEM ***********************************
    A struct for storage of a parsing return.
**************************************************/
typedef struct _TepParse
{
    TepID               Command;    /* id of the command being used */
    TepID               Clue;       /* id of the clue being referenced */
    TepID               Char1;      /* id of the first character referenced */
    TepID               Char2;      /* id of the second character referenced */
    TepID               Reserved;   /* contextual meaning. This object is reserved for future use, and is not needed for any functionality yet. */
} TepParse;


/* POINTER TO PARSING ITEM ************************
    Storage of a pointer to a parsing return
**************************************************/
typedef TepParse *pTepParse;


/***  CONVERSATION STRUCTS  **************************************************
*****************************************************************************/
/* CONVERSATION ITEM ******************************
    A struct for storage of a single conversation
    piece.
**************************************************/
typedef struct _TepConvData
{
    TepID                   id;             /* conversation id */
    TepID                   attachedto;     /* ID of object attached to */
    pChar                   details;        /* null-termed string of its data */
    struct _TepConvData     *NEXT;          /* pointer to next ConvData */
} TepConvData; /* struct _TepConvData */


/* POINTER TO CONVERSATION ITEM ******************/
typedef TepConvData *pTepConvData;


/***  ROOM STRUCTS  **********************************************************
*****************************************************************************/
/* ROOM STRUCTURE *********************************
    A struct for storage of rooms
**************************************************/
typedef struct _TepRoom
{
    TepID               id;         /* value 0 is an undefined value. These are to be unique values. */
    pChar               name;       /* pointer to null-termed string */
    pChar               desc;       /* pointer to object's description data */
    TepCmds             cmd;        /* bit fields of allowed commands of this object. */
    pTepIDUnNode        rooms;      /* pointer to linked list of id's for connected rooms */
    pTepIDUnNode        objs;       /* pointer to linked list of id's for objects in room */
    struct _TepRoom     *NEXT;      /* pointer to the next node */
} TepRoom; /* struct _TepRoom */


/* POINTER TO A ROOM STRUCTURE *******************/
typedef TepRoom *pTepRoom;


/* DYNAMIC ROOM DATA ******************************
    A struct for storage of all current dynamic map
    data.
**************************************************/
typedef struct _TepDynRoom
{
    TepID               asRoomID;   /* the id number of the room this object is associated with */
    pTepRoom            asRoom;     /* pointer to all data associated room */
    pTepIDUnNode        people;     /* pointer to linked list of id's for characters in room */
    pTepIDUnNode        objs;       /* pointer to linked list of id's for objects in room */
    struct _TepDynRoom  *NEXT;      /* pointer to next dynamic room data struct */
} TepDynRoom; /* struct _TepDynRoom */


/* POINTER TO DYNAMIC ROOM DATA ******************/
typedef TepDynRoom *pTepDynRoom;


/***  CLUES  *****************************************************************
*****************************************************************************/
/* CLUE STRUCT ************************************
    A struct for storage of a single clue.
**************************************************/
typedef struct _TepClueData
{
    TepID                   id;         /* conversation id */
    pTepIDUnNode            attachedto; /* ID of object attached to */
	//TepID					attachedto; /* ID of object attached to */
	pChar					keyword;	/* null-termed string containing the keywords - added by matt on 2/26/01 */
    pChar                   details;    /* null-termed string of its data */
    struct _TepClueData     *NEXT;      /* pointer to next ClueData */
} TepClueData; /* struct _TepClueData */


/* POINTER TO CLUE *******************************/
typedef TepClueData *pTepClueData;


/***  CHARACTER STRUCTS  *****************************************************
*****************************************************************************/
/* DOES CHARACTER LIKE PLAYER? ********************
    A nibble (16 values) illustrating the level
    of fondness for the player a character has.
**************************************************/
typedef uChar TepCharLikePlayer;


/* POINTER TO "LIKE PLAYER" ***********************
    Pointer to "does character like player?"
**************************************************/
typedef TepCharLikePlayer *pTepCharLikePlayer;


/* PERSONALITY MATRIX *****************************
    Bit field personality matrix
**************************************************/
typedef ULONG TepCharPersonality;


/* POINTER TO COMMANDS ALLOWED STRUCT *************
    Pointer to bit level personality matrix
**************************************************/
typedef TepCharPersonality *pTepCharPersonality;


/* WHACKLIST STRUCTURE ****************************
    Whack list structure
**************************************************/
typedef struct _TepWhack
{
    TepID               character;  /* character ths pertains to */
    USHORT              value;      /* relative value attached to this character */
    struct _TepWhack    *NEXT;      /* pointer to next whack list data struct */
} TepWhack;


/* POINTER TO WHACKLIST STRUCT ********************
    Pointer to Whack List structure
**************************************************/
typedef TepWhack *pTepWhack;


/* CHARACTER STRUCT *******************************
    A struct for storage of all character data,
    except the player's.
**************************************************/
typedef struct _TepChar
{
    TepID                   id;                     /* unique id number of this character */
    pChar                   name;                   /* pointer to a null-termed string containing the character's name */
    pChar                   desc;                   /* pointer to a null-termed string containing the character's description */
    pTepConvData            conv;                   /* pointer to charcter's conversation matrix */
    pTepWhack               whacklist;              /* pointer to an bidirectional id list containing the character's whack data */
    pTepIDUnNode			clues;					/* pointer to a list of IDs of clues attached to the character - edited by matt on 3/3/01 */
		// old clues - pTepClueData            clues;                  /* pointer to a clue data structure containing actual attached clues are visible to the player upon description */
	pTepIDUnNode            knownData;              /* pointer to an unidirectional id list containing the data (clues, deaths and such) that the character knows about. This is used, so that the player cannot show a give character the same thing over and over to change their whack list */
    TepCharLikePlayer		like;                   /* nibble containing levels of emotion toward the player */
    pTepCharPersonality     ident;                  /* bit field containing flags for personality traits */
    pTepIDUnNode            locations;              /* list of possible locations to find the character */
    uChar                   complimentCounter;      /* preset to a random number in each character at beginning of day, (between 1-3) */
    struct _TepChar         *NEXT;                  /* pointer to next node */
} TepChar; /* struct _TepChar */


/* POINTER TO CHARACTER STRUCT *******************/
typedef TepChar *pTepChar;


/***  EVIDENCE STRUCT  *******************************************************
*****************************************************************************/
/* EVIDENCE ***************************************
    A struct for storage of evidence data
**************************************************/
typedef struct _TepPlayerEvidence
{
    TepID                       clue;           /* clue id this is for */
    TepID                       attachedTo;     /* id num of object that evidence is attached to */
    struct _TepPlayerEvidence   *NEXT;          /* pointer to next node */
} TepPlayerEvidence; /* struct _TepPlayerEvidence */


/* POINTER TO EVIDENCE ***************************/
typedef TepPlayerEvidence *pTepPlayerEvidence;


/***  PLAYER STRUCT  *********************************************************
*****************************************************************************/
/* PLAYER *****************************************
    A struct for storage of the player's
    information.
**************************************************/
typedef struct _TepPlayer
{
    pChar                       name;                   /* player's name */
    pTepPlayerEvidence          evidenceAgainst;        /* pointer to list of current evidence against a character */
    pTepPlayerEvidence          evidenceGeneric;        /* pointer to list of current evidence that has yet to be placed against someone */
    pTepPlayerEvidence          oldEvidenceAgainst;     /* pointer to list of past evidence against a character */
    pTepPlayerEvidence          oldEvidenceGeneric;     /* pointer to list of past evidence that has yet to be placed against someone */
} TepPlayer; /* struct _TepPlayer */


/* POINTER TO PLAYER *****************************/
typedef TepPlayer *pTepPlayer;


/***  DEATH STRUCT  **********************************************************
*****************************************************************************/
/* DEATH ******************************************
    A struct for storage of deaths
**************************************************/
typedef struct _TepDeath
{
    TepID               id;             /* id of the death */
    TepID				location;		/* id of the location if any */
    pChar               desc;           /* description of death */
    pTepConvData        conv;           /* pointer to conversation structures */
	//pTepIDUnNode		clues;			/* pointer to a list of IDs of clues that pertain to this death */
	pTepClueData        clues;          /* pointer to clue data structures */
    struct _TepDeath    *NEXT;          /* pointer to next node */
} TepDeath; /* struct _TepDeath */


/* POINTER TO DEATH ******************************/
typedef TepDeath *pTepDeath;


/***  MURDER STRUCT  *********************************************************
*****************************************************************************/
/* CURRENT MURDER *********************************
    A struct for storage of the current murder
**************************************************/
typedef struct _TepCurMurder
{
    TepID	                victim;     /* ID of the victim */
    TepID	                murderer;   /* ID of the murderer */
    pTepDeath               death;      /* pointer to a death construct */
    struct _TepCurMurder    *NEXT;      /* pointer to next node */
} TepCurMurder; /* struct _TepCurMurder */


/* POINTER TO CURRENT MURDER *********************/
typedef TepCurMurder *pTepCurMurder;


/***  GAME STATE  ************************************************************
*****************************************************************************/
/* GAME STATE *************************************
    A structure devoted to storing the current
    game's state. This is where most functions
    will get their data from.
**************************************************/
typedef struct _TepCurState
{
    USHORT              turnsLeft;          /* the number of turns left in current day */
    BOOL                fNeedSave;          /* does the player need to save the game if they exit */
    pTepRoom            currentRoom;        /* pointer to current room player is in */
    pTepIDUnNode        killersList;        /* pointer to a unidirectional list of the Servants of Rametep */
    pTepCurMurder       pastMurders;        /* pointer to a unidirectional list of used murders */
    pTepChar            liveList;           /* pointer to a unidirectional list of live characters */
    pTepPlayer          player;             /* pointer to the player's data */
    pTepCurMurder       currentMurder;      /* pointer to the current murder and its affiliated data */
    pTepDynRoom         mapLite;            /* pointer to a bidirectional list of dynamic map content */
    pTepRoom            FullMap;            /* pointer to full version of entire map */
	pTepDataString		keywords;			/* pointer to a list of keywords and thair matching IDs */
	pTepConvData		genConv;			/* pointer to a list of generic character conversation data */
	//pTepClueData		clueList;			/* pointer to a list of clues used in the game - added by matt 2/26/01 */
} TepCurState; /* struct _TepCurState */


/* POINTER TO GAME STATE *************************/
typedef TepCurState *pTepCurState;

/***  POINTER LIST STRUCTS ***************************************************
*****************************************************************************/
/* POINTER LIST STRUCT ****************************
    A struct for storage of pointers to other
	structs. This structure is to be used primarily
	to hold return values of searches.
**************************************************/
typedef struct _TepPList
{
	void				*pNode;				/* pointer to some node in another list */
	struct	_TepPList	*NEXT;				/* pointer to the next node in this list */
}TepPList;


/* POINTER TO POINTER LIST STRUCT ****************/
typedef TepPList *pTepPList;


#endif

/* EOF */
